@@ -50,11 +50,10 @@
 #include "cally-root.h"
 #include "cally-stage.h"
 
+#define DEFAULT_PASSWORD_CHAR '*'
+
 /* atkutil.h */
 
-static guint                 cally_util_add_global_event_listener    (GSignalEmissionHook listener,
-                                                                      const gchar*        event_type);
-static void                  cally_util_remove_global_event_listener (guint remove_listener);
 static guint                 cally_util_add_key_event_listener	    (AtkKeySnoopFunc listener,
                                                                      gpointer        data);
 static void                  cally_util_remove_key_event_listener    (guint remove_listener);
@@ -63,11 +62,6 @@ static const gchar *         cally_util_get_toolkit_name		    (void);
 static const gchar *         cally_util_get_toolkit_version          (void);
 
 /* private */
-static void		     _listener_info_destroy                  (gpointer data);
-static guint                 add_listener	                     (GSignalEmissionHook  listener,
-                                                                      const gchar         *object_type,
-                                                                      const gchar         *signal,
-                                                                      const gchar         *hook_data);
 static void                  cally_util_simulate_snooper_install     (void);
 static void                  cally_util_simulate_snooper_remove      (void);
 static gboolean              cally_key_snooper                       (ClutterActor *actor,
@@ -85,21 +79,11 @@ static gboolean              notify_hf                               (gpointer k
 static void                  insert_hf                               (gpointer key,
                                                                       gpointer value,
                                                                       gpointer data);
-static AtkKeyEventStruct *  atk_key_event_from_clutter_event_key     (ClutterKeyEvent *event);
-
 
 /* This is just a copy of the Gail one, a shared library or place to
    define it could be a good idea. */
-typedef struct _CallyUtilListenerInfo CallyUtilListenerInfo;
 typedef struct _CallyKeyEventInfo CallyKeyEventInfo;
 
-struct _CallyUtilListenerInfo
-{
-  gint key;
-  guint signal_id;
-  gulong hook_id;
-};
-
 struct _CallyKeyEventInfo
 {
   AtkKeySnoopFunc listener;
@@ -107,9 +91,7 @@ struct _CallyKeyEventInfo
 };
 
 static AtkObject* root = NULL;
-static GHashTable *listener_list = NULL;
 static GHashTable *key_listener_list = NULL;
-static gint listener_idx = 1;
 
 
 G_DEFINE_TYPE (CallyUtil, cally_util, ATK_TYPE_UTIL);
@@ -123,8 +105,6 @@ cally_util_class_init (CallyUtilClass *klass)
   data = g_type_class_peek (ATK_TYPE_UTIL);
   atk_class = ATK_UTIL_CLASS (data);
 
-  atk_class->add_global_event_listener    = cally_util_add_global_event_listener;
-  atk_class->remove_global_event_listener = cally_util_remove_global_event_listener;
   atk_class->add_key_event_listener       = cally_util_add_key_event_listener;
   atk_class->remove_key_event_listener    = cally_util_remove_key_event_listener;
   atk_class->get_root                     = cally_util_get_root;
@@ -136,9 +116,6 @@ cally_util_class_init (CallyUtilClass *klass)
      class methods will access this instance. This will be a good
      future enhancement, meanwhile, just using the same *working*
      implementation used on GailUtil */
-
-  listener_list = g_hash_table_new_full (g_int_hash, g_int_equal, NULL,
-                                         _listener_info_destroy);
 }
 
 static void
@@ -170,70 +147,11 @@ cally_util_get_toolkit_version (void)
   return CLUTTER_VERSION_S;
 }
 
-
-static guint
-cally_util_add_global_event_listener (GSignalEmissionHook listener,
-                                      const gchar *event_type)
-{
-  guint rc = 0;
-  gchar **split_string;
-
-  split_string = g_strsplit (event_type, ":", 3);
-
-  if (g_strv_length (split_string) == 3)
-    rc = add_listener (listener, split_string[1], split_string[2], event_type);
-
-  g_strfreev (split_string);
-
-  return rc;
-}
-
-static void
-cally_util_remove_global_event_listener (guint remove_listener)
-{
-  if (remove_listener > 0)
-    {
-      CallyUtilListenerInfo *listener_info;
-      gint tmp_idx = remove_listener;
-
-      listener_info = (CallyUtilListenerInfo *)
-        g_hash_table_lookup(listener_list, &tmp_idx);
-
-      if (listener_info != NULL)
-        {
-          /* Hook id of 0 and signal id of 0 are invalid */
-          if (listener_info->hook_id != 0 && listener_info->signal_id != 0)
-            {
-              /* Remove the emission hook */
-              g_signal_remove_emission_hook(listener_info->signal_id,
-                                            listener_info->hook_id);
-
-              /* Remove the element from the hash */
-              g_hash_table_remove(listener_list, &tmp_idx);
-            }
-          else
-            {
-              g_warning("Invalid listener hook_id %ld or signal_id %d\n",
-                        listener_info->hook_id, listener_info->signal_id);
-            }
-        }
-      else
-        {
-          g_warning("No listener with the specified listener id %d", 
-                    remove_listener);
-        }
-    }
-  else
-    {
-      g_warning("Invalid listener_id %d", remove_listener);
-    }
-}
-
 static guint
 cally_util_add_key_event_listener (AtkKeySnoopFunc  listener,
                                    gpointer         data)
 {
-  static guint key=0;
+  static guint key = 1;
   CallyKeyEventInfo *event_info = NULL;
 
   if (!key_listener_list)
@@ -261,63 +179,14 @@ cally_util_remove_key_event_listener (guint remove_listener)
 
   if (g_hash_table_size (key_listener_list) == 0)
     {
+      g_hash_table_destroy (key_listener_list);
+      key_listener_list = NULL;
       cally_util_simulate_snooper_remove ();
     }
 }
 
 /* ------------------------------ PRIVATE FUNCTIONS ------------------------- */
 
-static void
-_listener_info_destroy (gpointer data)
-{
-  g_free(data);
-}
-
-static guint
-add_listener (GSignalEmissionHook listener,
-              const gchar         *object_type,
-              const gchar         *signal_name,
-              const gchar         *hook_data)
-{
-  GType type;
-  guint signal_id;
-  gint  rc = 0;
-
-  type = g_type_from_name (object_type);
-  if (type)
-    {
-      signal_id  = g_signal_lookup (signal_name, type);
-      if (signal_id > 0)
-        {
-          CallyUtilListenerInfo *listener_info;
-
-          rc = listener_idx;
-
-          listener_info = g_new (CallyUtilListenerInfo, 1);
-          listener_info->key = listener_idx;
-          listener_info->hook_id =
-            g_signal_add_emission_hook (signal_id, 0, listener,
-                                        g_strdup (hook_data),
-                                        (GDestroyNotify) g_free);
-          listener_info->signal_id = signal_id;
-
-	  g_hash_table_insert(listener_list, &(listener_info->key), listener_info);
-          listener_idx++;
-        }
-      else
-        {
-          /* This is mainly because some "window:xxx" methods not
-             implemented on CallyStage */
-          g_debug ("Signal type %s not supported\n", signal_name);
-        }
-    }
-  else
-    {
-      g_warning("Invalid object type %s\n", object_type);
-    }
-  return rc;
-}
-
 /* Trying to emulate gtk_key_snooper install (a kind of wrapper). This
    could be implemented without it, but I will maintain it in this
    way, so if in the future clutter implements it natively it would be
@@ -380,7 +249,8 @@ cally_util_simulate_snooper_remove (void)
 }
 
 static AtkKeyEventStruct *
-atk_key_event_from_clutter_event_key (ClutterKeyEvent *clutter_event)
+atk_key_event_from_clutter_event_key (ClutterKeyEvent *clutter_event,
+                                      gunichar         password_char)
 {
   AtkKeyEventStruct *atk_event = g_new0 (AtkKeyEventStruct, 1);
   gunichar key_unichar;
@@ -398,7 +268,10 @@ atk_key_event_from_clutter_event_key (ClutterKeyEvent *clutter_event)
       return NULL;
     }
 
-  atk_event->state = clutter_event->modifier_state;
+  if (password_char)
+    atk_event->state = 0;
+  else
+    atk_event->state = clutter_event->modifier_state;
 
   /* We emit the clutter keyval. This is not exactly the one expected
      by AtkKeyEventStruct, as it expects a Gdk-like event, with the
@@ -406,7 +279,10 @@ atk_key_event_from_clutter_event_key (ClutterKeyEvent *clutter_event)
      that on the AT application.
      More information: Bug 1952 and bug 2072
   */
-  atk_event->keyval = clutter_event->keyval;
+  if (password_char)
+    atk_event->keyval = clutter_unicode_to_keysym (password_char);
+  else
+    atk_event->keyval = clutter_event->keyval;
 
   /* It is expected to store a key defining string here (ie "Space" in
      case you press a space). Anyway, there are no function on clutter
@@ -417,7 +293,10 @@ atk_key_event_from_clutter_event_key (ClutterKeyEvent *clutter_event)
      More information: Bug 1952 and 2072
   */
 
-  key_unichar = clutter_event_get_key_unicode ((ClutterEvent *) clutter_event);
+  if (password_char)
+    key_unichar = password_char;
+  else
+    key_unichar = clutter_event_get_key_unicode ((ClutterEvent *) clutter_event);
 
   if (g_unichar_validate (key_unichar) && !g_unichar_iscntrl (key_unichar))
     {
@@ -433,7 +312,15 @@ atk_key_event_from_clutter_event_key (ClutterKeyEvent *clutter_event)
 
   atk_event->length = 0;
 
-  atk_event->keycode = clutter_event->hardware_keycode;
+  /* Computing the hardware keycode from the password-char is
+     difficult. But we are in a password situation. We are already a
+     unichar that it is not the original one. Providing a "almost
+     real" keycode is irrelevant */
+  if (password_char)
+    atk_event->keycode = 0;
+  else
+    atk_event->keycode = clutter_event->hardware_keycode;
+
   atk_event->timestamp = clutter_event->time;
 
 #ifdef CALLY_DEBUG
@@ -466,6 +353,37 @@ insert_hf (gpointer key, gpointer value, gpointer data)
   g_hash_table_insert (new_table, key, value);
 }
 
+
+/*
+ * 0 if the key of that event is visible, in other case the password
+ * char
+ */
+static gunichar
+check_key_visibility (ClutterEvent *event)
+{
+  ClutterKeyEvent *key_event = (ClutterKeyEvent *)event;
+  AtkObject *accessible = clutter_actor_get_accessible (key_event->source);
+
+  g_return_val_if_fail (accessible != NULL, 0);
+
+  if (atk_object_get_role (accessible) != ATK_ROLE_PASSWORD_TEXT)
+    return 0;
+
+  /* If it is a clutter text, we use his password char.  Note that
+     although at Clutter toolkit itself, only ClutterText exposes a
+     password role, nothing prevents on any derived toolkit (like st)
+     to create a new actor that can behave like a password entry. And
+     the key event will still be emitted here. Although in that case
+     we would lose any password char from the derived toolkit, it is
+     still better fill this with a default unichar that the original
+     one */
+
+  if (CLUTTER_IS_TEXT (key_event->source))
+    return clutter_text_get_password_char (CLUTTER_TEXT (key_event->source));
+  else
+    return DEFAULT_PASSWORD_CHAR;
+}
+
 static gboolean
 cally_key_snooper (ClutterActor *actor,
                    ClutterEvent *event,
@@ -473,6 +391,7 @@ cally_key_snooper (ClutterActor *actor,
 {
   AtkKeyEventStruct *key_event = NULL;
   gint consumed = 0;
+  gunichar password_char = 0;
 
   /* filter key events */
   if ((event->type != CLUTTER_KEY_PRESS) && (event->type != CLUTTER_KEY_RELEASE))
@@ -480,19 +399,22 @@ cally_key_snooper (ClutterActor *actor,
       return FALSE;
     }
 
+  password_char = check_key_visibility (event);
+
   if (key_listener_list)
     {
       GHashTable *new_hash = g_hash_table_new (NULL, NULL);
 
       g_hash_table_foreach (key_listener_list, insert_hf, new_hash);
-      key_event = atk_key_event_from_clutter_event_key ((ClutterKeyEvent *)event);
+      key_event = atk_key_event_from_clutter_event_key ((ClutterKeyEvent *)event,
+                                                        password_char);
       /* func data is inside the hash table */
       consumed = g_hash_table_foreach_steal (new_hash, notify_hf, key_event);
       g_hash_table_destroy (new_hash);
-    }
 
-  g_free (key_event->string);
-  g_free (key_event);
+      g_free (key_event->string);
+      g_free (key_event);
+    }
 
   return (consumed ? 1 : 0);
 }
