@@ -38,7 +38,7 @@
  *
  * <informalexample id="canvas-example">
  *   <programlisting>
- * <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" parse="text" href="../../../../tests/interactive/test-canvas.c">
+ * <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" parse="text" href="../../../../examples/canvas.c">
  *   <xi:fallback>FIXME: MISSING XINCLUDE CONTENT</xi:fallback>
  * </xi:include>
  *   </programlisting>
@@ -59,6 +59,7 @@
 #define CLUTTER_ENABLE_EXPERIMENTAL_API
 
 #include "clutter-backend.h"
+#include "clutter-cairo.h"
 #include "clutter-color.h"
 #include "clutter-content-private.h"
 #include "clutter-marshal.h"
@@ -100,6 +101,7 @@ static guint canvas_signals[LAST_SIGNAL] = { 0, };
 static void clutter_content_iface_init (ClutterContentIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (ClutterCanvas, clutter_canvas, G_TYPE_OBJECT,
+                         G_ADD_PRIVATE (ClutterCanvas)
                          G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_CONTENT,
                                                 clutter_content_iface_init))
 
@@ -150,19 +152,29 @@ clutter_canvas_set_property (GObject      *gobject,
   switch (prop_id)
     {
     case PROP_WIDTH:
-      if (priv->width != g_value_get_int (value))
-        {
-          priv->width = g_value_get_int (value);
-          clutter_content_invalidate (CLUTTER_CONTENT (gobject));
-        }
+      {
+        gint new_size = g_value_get_int (value);
+
+        if (priv->width != new_size)
+          {
+            priv->width = new_size;
+
+            clutter_content_invalidate (CLUTTER_CONTENT (gobject));
+          }
+      }
       break;
 
     case PROP_HEIGHT:
-      if (priv->height != g_value_get_int (value))
-        {
-          priv->height = g_value_get_int (value);
-          clutter_content_invalidate (CLUTTER_CONTENT (gobject));
-        }
+      {
+        gint new_size = g_value_get_int (value);
+
+        if (priv->height != new_size)
+          {
+            priv->height = new_size;
+
+            clutter_content_invalidate (CLUTTER_CONTENT (gobject));
+          }
+      }
       break;
 
     default:
@@ -200,8 +212,6 @@ clutter_canvas_class_init (ClutterCanvasClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (ClutterCanvasPrivate));
-
   /**
    * ClutterCanvas:width:
    *
@@ -275,9 +285,7 @@ clutter_canvas_class_init (ClutterCanvasClass *klass)
 static void
 clutter_canvas_init (ClutterCanvas *self)
 {
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, CLUTTER_TYPE_CANVAS,
-                                            ClutterCanvasPrivate);
-
+  self->priv = clutter_canvas_get_instance_private (self);
   self->priv->width = -1;
   self->priv->height = -1;
 }
@@ -294,6 +302,7 @@ clutter_canvas_paint_content (ClutterContent   *content,
   ClutterColor color;
   guint8 paint_opacity;
   ClutterScalingFilter min_f, mag_f;
+  ClutterContentRepeat repeat;
 
   if (self->priv->buffer == NULL)
     return;
@@ -307,6 +316,7 @@ clutter_canvas_paint_content (ClutterContent   *content,
   clutter_actor_get_content_box (actor, &box);
   paint_opacity = clutter_actor_get_paint_opacity (actor);
   clutter_actor_get_content_scaling_filters (actor, &min_f, &mag_f);
+  repeat = clutter_actor_get_content_repeat (actor);
 
   color.red = paint_opacity;
   color.green = paint_opacity;
@@ -317,7 +327,24 @@ clutter_canvas_paint_content (ClutterContent   *content,
   cogl_object_unref (texture);
 
   clutter_paint_node_set_name (node, "Canvas");
-  clutter_paint_node_add_rectangle (node, &box);
+
+  if (repeat == CLUTTER_REPEAT_NONE)
+    clutter_paint_node_add_rectangle (node, &box);
+  else
+    {
+      float t_w = 1.f, t_h = 1.f;
+
+      if ((repeat & CLUTTER_REPEAT_X_AXIS) != FALSE)
+        t_w = (box.x2 - box.x1) / cogl_texture_get_width (texture);
+
+      if ((repeat & CLUTTER_REPEAT_Y_AXIS) != FALSE)
+        t_h = (box.y2 - box.y1) / cogl_texture_get_height (texture);
+
+      clutter_paint_node_add_texture_rectangle (node, &box,
+                                                0.f, 0.f,
+                                                t_w, t_h);
+    }
+
   clutter_paint_node_add_child (root, node);
   clutter_paint_node_unref (node);
 }
@@ -333,7 +360,7 @@ clutter_canvas_emit_draw (ClutterCanvas *self)
   gboolean res;
   cairo_t *cr;
 
-  g_assert (priv->width >= 0 && priv->width >= 0);
+  g_assert (priv->width > 0 && priv->width > 0);
 
   if (priv->buffer == NULL)
     {
@@ -382,6 +409,15 @@ clutter_canvas_emit_draw (ClutterCanvas *self)
                  cr, priv->width, priv->height,
                  &res);
 
+#ifdef CLUTTER_ENABLE_DEBUG
+  if (_clutter_diagnostic_enabled () && cairo_status (cr))
+    {
+      g_warning ("Drawing failed for <ClutterCanvas>[%p]: %s",
+                 self,
+                 cairo_status_to_string (cairo_status (cr)));
+    }
+#endif
+
   self->priv->cr = NULL;
   cairo_destroy (cr);
 
@@ -411,7 +447,7 @@ clutter_canvas_invalidate (ClutterContent *content)
       priv->buffer = NULL;
     }
 
-  if (priv->width < 0 || priv->height < 0)
+  if (priv->width <= 0 || priv->height <= 0)
     return;
 
   clutter_canvas_emit_draw (self);
@@ -465,28 +501,14 @@ clutter_canvas_new (void)
   return g_object_new (CLUTTER_TYPE_CANVAS, NULL);
 }
 
-/**
- * clutter_canvas_set_size:
- * @canvas: a #ClutterCanvas
- * @width: the width of the canvas, in pixels
- * @height: the height of the canvas, in pixels
- *
- * Sets the size of the @canvas.
- *
- * This function will cause the @canvas to be invalidated.
- *
- * Since: 1.10
- */
-void
-clutter_canvas_set_size (ClutterCanvas *canvas,
-                         int            width,
-                         int            height)
+static gboolean
+clutter_canvas_invalidate_internal (ClutterCanvas *canvas,
+                                    int            width,
+                                    int            height)
 {
-  GObject *obj;
   gboolean width_changed = FALSE, height_changed = FALSE;
-
-  g_return_if_fail (CLUTTER_IS_CANVAS (canvas));
-  g_return_if_fail (width >= -1 && height >= -1);
+  gboolean res = FALSE;
+  GObject *obj;
 
   obj = G_OBJECT (canvas);
 
@@ -509,7 +531,48 @@ clutter_canvas_set_size (ClutterCanvas *canvas,
     }
 
   if (width_changed || height_changed)
-    clutter_content_invalidate (CLUTTER_CONTENT (canvas));
+    {
+      clutter_content_invalidate (CLUTTER_CONTENT (canvas));
+      res = TRUE;
+    }
 
   g_object_thaw_notify (obj);
+
+  return res;
+}
+
+/**
+ * clutter_canvas_set_size:
+ * @canvas: a #ClutterCanvas
+ * @width: the width of the canvas, in pixels
+ * @height: the height of the canvas, in pixels
+ *
+ * Sets the size of the @canvas, and invalidates the content.
+ *
+ * This function will cause the @canvas to be invalidated only
+ * if the size of the canvas surface has changed.
+ *
+ * If you want to invalidate the contents of the @canvas when setting
+ * the size, you can use the return value of the function to conditionally
+ * call clutter_content_invalidate():
+ *
+ * |[
+ *   if (!clutter_canvas_set_size (canvas, width, height))
+ *     clutter_content_invalidate (CLUTTER_CONTENT (canvas));
+ * ]|
+ *
+ * Return value: this function returns %TRUE if the size change
+ *   caused a content invalidation, and %FALSE otherwise
+ *
+ * Since: 1.10
+ */
+gboolean
+clutter_canvas_set_size (ClutterCanvas *canvas,
+                         int            width,
+                         int            height)
+{
+  g_return_val_if_fail (CLUTTER_IS_CANVAS (canvas), FALSE);
+  g_return_val_if_fail (width >= -1 && height >= -1, FALSE);
+
+  return clutter_canvas_invalidate_internal (canvas, width, height);
 }
