@@ -57,7 +57,7 @@
  * <example id="drag-action-example">
  *   <title>A simple draggable actor</title>
  *   <programlisting>
- * <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" parse="text" href="../../../../tests/interactive/test-drag.c">
+ * <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" parse="text" href="../../../../examples/drag-action.c">
  *   <xi:fallback>FIXME: MISSING XINCLUDE CONTENT</xi:fallback>
  * </xi:include>
  *   </programlisting>
@@ -91,9 +91,12 @@ struct _ClutterDragActionPrivate
   gint y_drag_threshold;
   ClutterActor *drag_handle;
   ClutterDragAxis drag_axis;
+  ClutterRect drag_area;
 
   ClutterInputDevice *device;
+  ClutterEventSequence *sequence;
   gulong button_press_id;
+  gulong touch_begin_id;
   gulong capture_id;
 
   gfloat press_x;
@@ -111,6 +114,7 @@ struct _ClutterDragActionPrivate
   guint emit_delayed_press    : 1;
   guint in_drag               : 1;
   guint motion_events_enabled : 1;
+  guint drag_area_set         : 1;
 };
 
 enum
@@ -121,6 +125,8 @@ enum
   PROP_Y_DRAG_THRESHOLD,
   PROP_DRAG_HANDLE,
   PROP_DRAG_AXIS,
+  PROP_DRAG_AREA,
+  PROP_DRAG_AREA_SET,
 
   PROP_LAST
 };
@@ -130,6 +136,7 @@ static GParamSpec *drag_props[PROP_LAST] = { NULL, };
 enum
 {
   DRAG_BEGIN,
+  DRAG_PROGRESS,
   DRAG_MOTION,
   DRAG_END,
 
@@ -143,7 +150,7 @@ static gboolean on_captured_event (ClutterActor      *stage,
                                    ClutterEvent      *event,
                                    ClutterDragAction *action);
 
-G_DEFINE_TYPE (ClutterDragAction, clutter_drag_action, CLUTTER_TYPE_ACTION);
+G_DEFINE_TYPE_WITH_PRIVATE (ClutterDragAction, clutter_drag_action, CLUTTER_TYPE_ACTION)
 
 static void
 get_drag_threshold (ClutterDragAction *action,
@@ -183,11 +190,18 @@ emit_drag_begin (ClutterDragAction *action,
   if (priv->stage != NULL)
     {
       clutter_stage_set_motion_events_enabled (priv->stage, FALSE);
-      _clutter_stage_add_drag_actor (priv->stage,
-                                     clutter_event_get_device (event),
-                                     priv->drag_handle != NULL
-                                       ? priv->drag_handle
-                                       : actor);
+      if (clutter_event_type (event) == CLUTTER_TOUCH_BEGIN)
+        _clutter_stage_add_touch_drag_actor (priv->stage,
+                                             clutter_event_get_event_sequence (event),
+                                             priv->drag_handle != NULL
+                                             ? priv->drag_handle
+                                             : actor);
+      else
+        _clutter_stage_add_pointer_drag_actor (priv->stage,
+                                               clutter_event_get_device (event),
+                                               priv->drag_handle != NULL
+                                               ? priv->drag_handle
+                                               : actor);
     }
 
   g_signal_emit (action, drag_signals[DRAG_BEGIN], 0,
@@ -205,6 +219,7 @@ emit_drag_motion (ClutterDragAction *action,
   ClutterActor *drag_handle = NULL;
   gfloat delta_x, delta_y;
   gfloat motion_x, motion_y;
+  gboolean can_emit_drag_motion = TRUE;
 
   clutter_event_get_coords (event, &priv->last_motion_x, &priv->last_motion_y);
   priv->last_motion_state = clutter_event_get_state (event);
@@ -260,9 +275,17 @@ emit_drag_motion (ClutterDragAction *action,
         return;
     }
 
-  g_signal_emit (action, drag_signals[DRAG_MOTION], 0,
+  g_signal_emit (action, drag_signals[DRAG_PROGRESS], 0,
                  actor,
-                 delta_x, delta_y);
+                 delta_x, delta_y,
+                 &can_emit_drag_motion);
+
+  if (can_emit_drag_motion)
+    {
+      g_signal_emit (action, drag_signals[DRAG_MOTION], 0,
+                     actor,
+                     delta_x, delta_y);
+    }
 }
 
 static void
@@ -272,6 +295,12 @@ emit_drag_end (ClutterDragAction *action,
 {
   ClutterDragActionPrivate *priv = action->priv;
 
+  /* ::drag-end may result in the destruction of the actor, which in turn
+   * will lead to the removal and finalization of the action, so we need
+   * to keep the action alive for the entire emission sequence
+   */
+  g_object_ref (action);
+
   /* if we have an event, update our own state, otherwise we'll
    * just use the currently stored state when emitting the ::drag-end
    * signal
@@ -283,6 +312,8 @@ emit_drag_end (ClutterDragAction *action,
       priv->last_motion_device = clutter_event_get_device (event);
     }
 
+  priv->in_drag = FALSE;
+
   /* we might not have emitted ::drag-begin yet */
   if (!priv->emit_delayed_press)
     g_signal_emit (action, drag_signals[DRAG_END], 0,
@@ -290,6 +321,9 @@ emit_drag_end (ClutterDragAction *action,
                    priv->last_motion_x, priv->last_motion_y,
                    priv->last_motion_state);
 
+  if (priv->stage == NULL)
+    goto out;
+
   /* disconnect the capture */
   if (priv->capture_id != 0)
     {
@@ -300,10 +334,21 @@ emit_drag_end (ClutterDragAction *action,
   clutter_stage_set_motion_events_enabled (priv->stage,
                                            priv->motion_events_enabled);
 
-  if (priv->last_motion_device != NULL)
-    _clutter_stage_remove_drag_actor (priv->stage, priv->last_motion_device);
+  if (priv->last_motion_device != NULL && event != NULL)
+    {
+      if (clutter_event_type (event) == CLUTTER_BUTTON_RELEASE)
+        _clutter_stage_remove_pointer_drag_actor (priv->stage,
+                                                  priv->last_motion_device);
+      else
+        _clutter_stage_remove_touch_drag_actor (priv->stage,
+                                                priv->sequence);
+    }
 
-  priv->in_drag = FALSE;
+out:
+  priv->last_motion_device = NULL;
+  priv->sequence = NULL;
+
+  g_object_unref (action);
 }
 
 static gboolean
@@ -313,17 +358,22 @@ on_captured_event (ClutterActor      *stage,
 {
   ClutterDragActionPrivate *priv = action->priv;
   ClutterActor *actor;
-  
+
   actor = clutter_actor_meta_get_actor (CLUTTER_ACTOR_META (action));
 
   if (!priv->in_drag)
     return CLUTTER_EVENT_PROPAGATE;
 
-  if (clutter_event_get_device (event) != priv->device)
+  if (clutter_event_get_device (event) != priv->device ||
+      clutter_event_get_event_sequence (event) != priv->sequence)
     return CLUTTER_EVENT_PROPAGATE;
 
   switch (clutter_event_type (event))
     {
+    case CLUTTER_TOUCH_UPDATE:
+      emit_drag_motion (action, actor, event);
+      break;
+
     case CLUTTER_MOTION:
       {
         ClutterModifierType mods = clutter_event_get_state (event);
@@ -339,6 +389,11 @@ on_captured_event (ClutterActor      *stage,
       }
       break;
 
+    case CLUTTER_TOUCH_END:
+    case CLUTTER_TOUCH_CANCEL:
+      emit_drag_end (action, actor, event);
+      break;
+
     case CLUTTER_BUTTON_RELEASE:
       if (priv->in_drag)
         emit_drag_end (action, actor, event);
@@ -358,9 +413,9 @@ on_captured_event (ClutterActor      *stage,
 }
 
 static gboolean
-on_button_press (ClutterActor      *actor,
-                 ClutterEvent      *event,
-                 ClutterDragAction *action)
+on_drag_begin (ClutterActor      *actor,
+               ClutterEvent      *event,
+               ClutterDragAction *action)
 {
   ClutterDragActionPrivate *priv = action->priv;
 
@@ -368,8 +423,24 @@ on_button_press (ClutterActor      *actor,
     return CLUTTER_EVENT_PROPAGATE;
 
   /* dragging is only performed using the primary button */
-  if (clutter_event_get_button (event) != CLUTTER_BUTTON_PRIMARY)
-    return CLUTTER_EVENT_PROPAGATE;
+  switch (clutter_event_type (event))
+    {
+    case CLUTTER_BUTTON_PRESS:
+      if (priv->sequence != NULL)
+        return CLUTTER_EVENT_PROPAGATE;
+      if (clutter_event_get_button (event) != CLUTTER_BUTTON_PRIMARY)
+        return CLUTTER_EVENT_PROPAGATE;
+      break;
+
+    case CLUTTER_TOUCH_BEGIN:
+      if (priv->sequence != NULL)
+        return CLUTTER_EVENT_PROPAGATE;
+      priv->sequence = clutter_event_get_event_sequence (event);
+      break;
+
+    default:
+      return CLUTTER_EVENT_PROPAGATE;
+    }
 
   if (priv->stage == NULL)
     priv->stage = CLUTTER_STAGE (clutter_actor_get_stage (actor));
@@ -416,9 +487,13 @@ clutter_drag_action_set_actor (ClutterActorMeta *meta,
 
       old_actor = clutter_actor_meta_get_actor (meta);
       if (old_actor != NULL)
-        g_signal_handler_disconnect (old_actor, priv->button_press_id);
-        
+        {
+          g_signal_handler_disconnect (old_actor, priv->button_press_id);
+          g_signal_handler_disconnect (old_actor, priv->touch_begin_id);
+        }
+
       priv->button_press_id = 0;
+      priv->touch_begin_id = 0;
     }
 
   if (priv->capture_id != 0)
@@ -435,13 +510,27 @@ clutter_drag_action_set_actor (ClutterActorMeta *meta,
   priv->in_drag = FALSE;
 
   if (actor != NULL)
-    priv->button_press_id = g_signal_connect (actor, "button-press-event",
-                                              G_CALLBACK (on_button_press),
-                                              meta);
+    {
+      priv->button_press_id = g_signal_connect (actor, "button-press-event",
+                                                G_CALLBACK (on_drag_begin),
+                                                meta);
+      priv->touch_begin_id = g_signal_connect (actor, "touch-event",
+                                               G_CALLBACK (on_drag_begin),
+                                               meta);
+    }
 
   CLUTTER_ACTOR_META_CLASS (clutter_drag_action_parent_class)->set_actor (meta, actor);
 }
 
+static gboolean
+clutter_drag_action_real_drag_progress (ClutterDragAction *action,
+                                        ClutterActor      *actor,
+                                        gfloat             delta_x,
+                                        gfloat             delta_y)
+{
+  return TRUE;
+}
+
 static void
 clutter_drag_action_real_drag_motion (ClutterDragAction *action,
                                       ClutterActor      *actor,
@@ -449,13 +538,27 @@ clutter_drag_action_real_drag_motion (ClutterDragAction *action,
                                       gfloat             delta_y)
 {
   ClutterActor *drag_handle;
+  gfloat x, y;
 
   if (action->priv->drag_handle != NULL)
     drag_handle = action->priv->drag_handle;
   else
     drag_handle = actor;
 
-  clutter_actor_move_by (drag_handle, delta_x, delta_y);
+  clutter_actor_get_position (drag_handle, &x, &y);
+
+  x += delta_x;
+  y += delta_y;
+
+  if (action->priv->drag_area_set)
+    {
+      ClutterRect *drag_area = &action->priv->drag_area;
+
+      x = CLAMP (x, drag_area->origin.x, drag_area->origin.x + drag_area->size.width);
+      y = CLAMP (y, drag_area->origin.y, drag_area->origin.y + drag_area->size.height);
+    }
+
+  clutter_actor_set_position (drag_handle, x, y);
 }
 
 static void
@@ -489,6 +592,10 @@ clutter_drag_action_set_property (GObject      *gobject,
       clutter_drag_action_set_drag_axis (action, g_value_get_enum (value));
       break;
 
+    case PROP_DRAG_AREA:
+      clutter_drag_action_set_drag_area (action, g_value_get_boxed (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
     }
@@ -530,6 +637,14 @@ clutter_drag_action_get_property (GObject    *gobject,
       g_value_set_enum (value, priv->drag_axis);
       break;
 
+    case PROP_DRAG_AREA:
+      g_value_set_boxed (value, &priv->drag_area);
+      break;
+
+    case PROP_DRAG_AREA_SET:
+      g_value_set_boolean (value, priv->drag_area_set);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
     }
@@ -540,8 +655,28 @@ clutter_drag_action_dispose (GObject *gobject)
 {
   ClutterDragActionPrivate *priv = CLUTTER_DRAG_ACTION (gobject)->priv;
 
+  /* if we're being disposed while a capture is still present, we
+   * need to reset the state we are currently holding
+   */
+  if (priv->last_motion_device != NULL)
+    {
+      _clutter_stage_remove_pointer_drag_actor (priv->stage,
+                                                priv->last_motion_device);
+      priv->last_motion_device = NULL;
+    }
+
+  if (priv->sequence != NULL)
+    {
+      _clutter_stage_remove_touch_drag_actor (priv->stage,
+                                              priv->sequence);
+      priv->sequence = NULL;
+    }
+
   if (priv->capture_id != 0)
     {
+      clutter_stage_set_motion_events_enabled (priv->stage,
+                                               priv->motion_events_enabled);
+
       if (priv->stage != NULL)
         g_signal_handler_disconnect (priv->stage, priv->capture_id);
 
@@ -555,12 +690,17 @@ clutter_drag_action_dispose (GObject *gobject)
 
       actor = clutter_actor_meta_get_actor (CLUTTER_ACTOR_META (gobject));
       if (actor != NULL)
-        g_signal_handler_disconnect (actor, priv->button_press_id);
+        {
+          g_signal_handler_disconnect (actor, priv->button_press_id);
+          g_signal_handler_disconnect (actor, priv->touch_begin_id);
+        }
 
       priv->button_press_id = 0;
+      priv->touch_begin_id = 0;
     }
 
   clutter_drag_action_set_drag_handle (CLUTTER_DRAG_ACTION (gobject), NULL);
+  clutter_drag_action_set_drag_area (CLUTTER_DRAG_ACTION (gobject), NULL);
 
   G_OBJECT_CLASS (clutter_drag_action_parent_class)->dispose (gobject);
 }
@@ -571,10 +711,9 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
   ClutterActorMetaClass *meta_class = CLUTTER_ACTOR_META_CLASS (klass);
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (ClutterDragActionPrivate));
-
   meta_class->set_actor = clutter_drag_action_set_actor;
 
+  klass->drag_progress = clutter_drag_action_real_drag_progress;
   klass->drag_motion = clutter_drag_action_real_drag_motion;
 
   /**
@@ -669,6 +808,37 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
                        CLUTTER_DRAG_AXIS_NONE,
                        CLUTTER_PARAM_READWRITE);
 
+  /**
+   * ClutterDragAction:drag-area:
+   *
+   * Constains the dragging action (or in particular, the resulting
+   * actor position) to the specified #ClutterRect, in parent's
+   * coordinates.
+   *
+   * Since: 1.12
+   */
+  drag_props[PROP_DRAG_AREA] =
+    g_param_spec_boxed ("drag-area",
+			P_("Drag Area"),
+			P_("Constrains the dragging to a rectangle"),
+			CLUTTER_TYPE_RECT,
+			CLUTTER_PARAM_READWRITE);
+
+  /**
+   * ClutterDragAction:drag-area-set:
+   *
+   * Whether the #ClutterDragAction:drag-area property has been set.
+   *
+   * Since: 1.12
+   */
+  drag_props[PROP_DRAG_AREA_SET] =
+    g_param_spec_boolean ("drag-area-set",
+			  P_("Drag Area Set"),
+			  P_("Whether the drag area is set"),
+			  FALSE,
+			  CLUTTER_PARAM_READABLE);
+
+
   gobject_class->set_property = clutter_drag_action_set_property;
   gobject_class->get_property = clutter_drag_action_get_property;
   gobject_class->dispose = clutter_drag_action_dispose;
@@ -707,7 +877,47 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
                   CLUTTER_TYPE_MODIFIER_TYPE);
 
   /**
-   * ClutterDragAction::drag-motion
+   * ClutterDragAction::drag-progress:
+   * @action: the #ClutterDragAction that emitted the signal
+   * @actor: the #ClutterActor attached to the action
+   * @delta_x: the X component of the distance between the press event
+   *   that began the dragging and the current position of the pointer,
+   *   as of the latest motion event
+   * @delta_y: the Y component of the distance between the press event
+   *   that began the dragging and the current position of the pointer,
+   *   as of the latest motion event
+   *
+   * The ::drag-progress signal is emitted for each motion event after
+   * the #ClutterDragAction::drag-begin signal has been emitted.
+   *
+   * The components of the distance between the press event and the
+   * latest motion event are computed in the actor's coordinate space,
+   * to take into account eventual transformations. If you want the
+   * stage coordinates of the latest motion event you can use
+   * clutter_drag_action_get_motion_coords().
+   *
+   * The default handler will emit #ClutterDragAction::drag-motion,
+   * if #ClutterDragAction::drag-progress emission returns %TRUE.
+   *
+   * Return value: %TRUE if the drag should continue, and %FALSE
+   *   if it should be stopped.
+   *
+   * Since: 1.12
+   */
+  drag_signals[DRAG_PROGRESS] =
+    g_signal_new (I_("drag-progress"),
+                  CLUTTER_TYPE_DRAG_ACTION,
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (ClutterDragActionClass, drag_progress),
+                  _clutter_boolean_continue_accumulator, NULL,
+                  _clutter_marshal_BOOLEAN__OBJECT_FLOAT_FLOAT,
+                  G_TYPE_BOOLEAN, 3,
+                  CLUTTER_TYPE_ACTOR,
+                  G_TYPE_FLOAT,
+                  G_TYPE_FLOAT);
+
+  /**
+   * ClutterDragAction::drag-motion:
    * @action: the #ClutterDragAction that emitted the signal
    * @actor: the #ClutterActor attached to the action
    * @delta_x: the X component of the distance between the press event
@@ -729,9 +939,9 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
    * The default handler of the signal will call clutter_actor_move_by()
    * either on @actor or, if set, of #ClutterDragAction:drag-handle using
    * the @delta_x and @delta_y components of the dragging motion. If you
-   * want to override the default behaviour, you can connect to this
-   * signal and call g_signal_stop_emission_by_name() from within your
-   * callback.
+   * want to override the default behaviour, you can connect to the
+   * #ClutterDragAction::drag-progress signal and return %FALSE from the
+   * handler.
    *
    * Since: 1.4
    */
@@ -780,8 +990,7 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
 static void
 clutter_drag_action_init (ClutterDragAction *self)
 {
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, CLUTTER_TYPE_DRAG_ACTION,
-                                            ClutterDragActionPrivate);
+  self->priv = clutter_drag_action_get_instance_private (self);
 }
 
 /**
@@ -884,12 +1093,14 @@ clutter_drag_action_get_drag_threshold (ClutterDragAction *action,
 }
 
 static void
-on_drag_handle_destroy (ClutterActor      *actor,
+on_drag_handle_destroy (ClutterActor      *handle,
                         ClutterDragAction *action)
 {
   ClutterDragActionPrivate *priv = action->priv;
+  ClutterActor *actor;
 
   /* make sure we reset the state */
+  actor = clutter_actor_meta_get_actor (CLUTTER_ACTOR_META (action));
   if (priv->in_drag)
     emit_drag_end (action, actor, NULL);
 
@@ -926,10 +1137,20 @@ clutter_drag_action_set_drag_handle (ClutterDragAction *action,
 
   priv->drag_handle = handle;
 
+  priv->transformed_press_x = priv->press_x;
+  priv->transformed_press_y = priv->press_y;
+
   if (priv->drag_handle != NULL)
-    g_signal_connect (priv->drag_handle, "destroy",
-                      G_CALLBACK (on_drag_handle_destroy),
-                      action);
+    {
+      clutter_actor_transform_stage_point (priv->drag_handle,
+                                           priv->press_x,
+                                           priv->press_y,
+					   &priv->transformed_press_x,
+					   &priv->transformed_press_y);
+      g_signal_connect (priv->drag_handle, "destroy",
+			G_CALLBACK (on_drag_handle_destroy),
+			action);
+    }
 
   g_object_notify_by_pspec (G_OBJECT (action), drag_props[PROP_DRAG_HANDLE]);
 }
@@ -1052,3 +1273,58 @@ clutter_drag_action_get_motion_coords (ClutterDragAction *action,
   if (motion_y)
     *motion_y = action->priv->last_motion_y;
 }
+
+/**
+ * clutter_drag_action_get_drag_area:
+ * @action: a #ClutterDragAction
+ * @drag_area: (out caller-allocates): a #ClutterRect to be filled
+ *
+ * Retrieves the "drag area" associated with @action, that
+ * is a #ClutterRect that constrains the actor movements,
+ * in parents coordinates.
+ *
+ * Returns: %TRUE if the actor is actually constrained (and thus
+ *          @drag_area is valid), %FALSE otherwise
+ */
+gboolean
+clutter_drag_action_get_drag_area (ClutterDragAction *action,
+				   ClutterRect       *drag_area)
+{
+  g_return_val_if_fail (CLUTTER_IS_DRAG_ACTION (action), FALSE);
+
+  if (drag_area != NULL)
+    *drag_area = action->priv->drag_area;
+  return action->priv->drag_area_set;
+}
+
+/**
+ * clutter_drag_action_set_drag_area:
+ * @action: a #ClutterDragAction
+ * @drag_area: (allow-none): a #ClutterRect
+ *
+ * Sets @drag_area to constrain the dragging of the actor associated
+ * with @action, so that it position is always within @drag_area, expressed
+ * in parent's coordinates.
+ * If @drag_area is %NULL, the actor is not constrained.
+ */
+void
+clutter_drag_action_set_drag_area (ClutterDragAction *action,
+				   const ClutterRect *drag_area)
+{
+  ClutterDragActionPrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_DRAG_ACTION (action));
+
+  priv = action->priv;
+
+  if (drag_area != NULL)
+    {
+      priv->drag_area = *drag_area;
+      priv->drag_area_set = TRUE;
+    }
+  else
+    priv->drag_area_set = FALSE;
+
+  g_object_notify_by_pspec (G_OBJECT (action), drag_props[PROP_DRAG_AREA_SET]);
+  g_object_notify_by_pspec (G_OBJECT (action), drag_props[PROP_DRAG_AREA]);
+}
