@@ -33,6 +33,8 @@
 #include "clutter-keysyms.h"
 #include "clutter-private.h"
 
+#include <math.h>
+
 /**
  * SECTION:clutter-event
  * @short_description: User and window system events
@@ -54,6 +56,11 @@ typedef struct _ClutterEventPrivate {
 
   gpointer platform_data;
 
+  ClutterModifierType button_state;
+  ClutterModifierType base_state;
+  ClutterModifierType latched_state;
+  ClutterModifierType locked_state;
+
   guint is_pointer_emulated : 1;
 } ClutterEventPrivate;
 
@@ -176,7 +183,9 @@ clutter_event_set_time (ClutterEvent *event,
  * clutter_event_get_state:
  * @event: a #ClutterEvent
  *
- * Retrieves the modifier state of the event.
+ * Retrieves the modifier state of the event. In case the window system
+ * supports reporting latched and locked modifiers, this function returns
+ * the effective state.
  *
  * Return value: the modifier state parameter, or 0
  *
@@ -263,6 +272,63 @@ clutter_event_set_state (ClutterEvent        *event,
     }
 }
 
+void
+_clutter_event_set_state_full (ClutterEvent        *event,
+			       ClutterModifierType  button_state,
+			       ClutterModifierType  base_state,
+			       ClutterModifierType  latched_state,
+			       ClutterModifierType  locked_state,
+			       ClutterModifierType  effective_state)
+{
+  ClutterEventPrivate *private = (ClutterEventPrivate*) event;
+
+  private->button_state = button_state;
+  private->base_state = base_state;
+  private->latched_state = latched_state;
+  private->locked_state = locked_state;
+
+  clutter_event_set_state (event, effective_state);
+}
+
+/**
+ * clutter_event_get_state_full:
+ * @event: a #ClutterEvent
+ * @button_state: (out) (allow-none): the pressed buttons as a mask
+ * @base_state: (out) (allow-none): the regular pressed modifier keys
+ * @latched_state: (out) (allow-none): the latched modifier keys (currently released but still valid for one key press/release)
+ * @locked_state: (out) (allow-none): the locked modifier keys (valid until the lock key is pressed and released again)
+ * @effective_state: (out) (allow-none): the logical OR of all the state bits above
+ *
+ * Retrieves the decomposition of the keyboard state into button, base,
+ * latched, locked and effective. This can be used to transmit to other
+ * applications, for example when implementing a wayland compositor.
+ *
+ * Since: 1.16
+ */
+void
+clutter_event_get_state_full (const ClutterEvent  *event,
+			      ClutterModifierType *button_state,
+			      ClutterModifierType *base_state,
+			      ClutterModifierType *latched_state,
+			      ClutterModifierType *locked_state,
+			      ClutterModifierType *effective_state)
+{
+  const ClutterEventPrivate *private = (const ClutterEventPrivate*) event;
+
+  g_return_if_fail (event != NULL);
+
+  if (button_state)
+    *button_state = private->button_state;
+  if (base_state)
+    *base_state = private->base_state;
+  if (latched_state)
+    *latched_state = private->latched_state;
+  if (locked_state)
+    *locked_state = private->locked_state;
+  if (effective_state)
+    *effective_state = clutter_event_get_state (event);
+}
+
 /**
  * clutter_event_get_coords:
  * @event: a #ClutterEvent
@@ -278,11 +344,34 @@ clutter_event_get_coords (const ClutterEvent *event,
                           gfloat             *x,
                           gfloat             *y)
 {
-  gfloat event_x, event_y;
+  ClutterPoint coords;
 
   g_return_if_fail (event != NULL);
 
-  event_x = event_y = 0;
+  clutter_event_get_position (event, &coords);
+
+  if (x != NULL)
+    *x = coords.x;
+
+  if (y != NULL)
+    *y = coords.y;
+}
+
+/**
+ * clutter_event_get_position:
+ * @event: a #ClutterEvent
+ * @position: a #ClutterPoint
+ *
+ * Retrieves the event coordinates as a #ClutterPoint.
+ *
+ * Since: 1.12
+ */
+void
+clutter_event_get_position (const ClutterEvent *event,
+                            ClutterPoint       *position)
+{
+  g_return_if_fail (event != NULL);
+  g_return_if_fail (position != NULL);
 
   switch (event->type)
     {
@@ -294,44 +383,35 @@ clutter_event_get_coords (const ClutterEvent *event,
     case CLUTTER_CLIENT_MESSAGE:
     case CLUTTER_DELETE:
     case CLUTTER_EVENT_LAST:
+      clutter_point_init (position, 0.f, 0.f);
       break;
 
     case CLUTTER_ENTER:
     case CLUTTER_LEAVE:
-      event_x = event->crossing.x;
-      event_y = event->crossing.y;
+      clutter_point_init (position, event->crossing.x, event->crossing.y);
       break;
 
     case CLUTTER_BUTTON_PRESS:
     case CLUTTER_BUTTON_RELEASE:
-      event_x = event->button.x;
-      event_y = event->button.y;
+      clutter_point_init (position, event->button.x, event->button.y);
       break;
 
     case CLUTTER_MOTION:
-      event_x = event->motion.x;
-      event_y = event->motion.y;
+      clutter_point_init (position, event->motion.x, event->motion.y);
       break;
 
     case CLUTTER_TOUCH_BEGIN:
     case CLUTTER_TOUCH_UPDATE:
     case CLUTTER_TOUCH_END:
     case CLUTTER_TOUCH_CANCEL:
-      event_x = event->touch.x;
-      event_y = event->touch.y;
+      clutter_point_init (position, event->touch.x, event->touch.y);
       break;
 
     case CLUTTER_SCROLL:
-      event_x = event->scroll.x;
-      event_y = event->scroll.y;
+      clutter_point_init (position, event->scroll.x, event->scroll.y);
       break;
     }
 
-  if (x)
-    *x = event_x;
-
-  if (y)
-    *y = event_y;
 }
 
 /**
@@ -821,7 +901,7 @@ clutter_event_set_key_code (ClutterEvent *event,
  *
  * Return value: The unicode value representing the key
  */
-guint32
+gunichar
 clutter_event_get_key_unicode (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, 0);
@@ -846,7 +926,7 @@ clutter_event_get_key_unicode (const ClutterEvent *event)
  */
 void
 clutter_event_set_key_unicode (ClutterEvent *event,
-                               guint32       key_unicode)
+                               gunichar      key_unicode)
 {
   g_return_if_fail (event != NULL);
   g_return_if_fail (event->type == CLUTTER_KEY_PRESS ||
@@ -999,6 +1079,8 @@ clutter_event_set_device (ClutterEvent       *event,
  * @event: a #ClutterEvent
  *
  * Retrieves the #ClutterInputDevice for the event.
+ * If you want the physical device the event originated from, use
+ * clutter_event_get_source_device().
  *
  * The #ClutterInputDevice structure is completely opaque and should
  * be cast to the platform-specific implementation.
@@ -1126,6 +1208,11 @@ clutter_event_copy (const ClutterEvent *event)
       new_real_event->source_device = real_event->source_device;
       new_real_event->delta_x = real_event->delta_x;
       new_real_event->delta_y = real_event->delta_y;
+      new_real_event->is_pointer_emulated = real_event->is_pointer_emulated;
+      new_real_event->base_state = real_event->base_state;
+      new_real_event->button_state = real_event->button_state;
+      new_real_event->latched_state = real_event->latched_state;
+      new_real_event->locked_state = real_event->locked_state;
     }
 
   device = clutter_event_get_device (event);
@@ -1352,12 +1439,12 @@ clutter_events_pending (void)
 guint32
 clutter_get_current_event_time (void)
 {
-  ClutterMainContext *context = _clutter_context_get_default ();
+  const ClutterEvent* event;
 
-  g_return_val_if_fail (context != NULL, FALSE);
+  event = clutter_get_current_event ();
 
-  if (context->last_event_time != 0)
-    return context->last_event_time;
+  if (event != NULL)
+    return clutter_event_get_time (event);
 
   return CLUTTER_CURRENT_TIME;
 }
@@ -1382,7 +1469,7 @@ clutter_get_current_event (void)
 
   g_return_val_if_fail (context != NULL, NULL);
 
-  return context->current_event;
+  return context->current_event != NULL ? context->current_event->data : NULL;
 }
 
 /**
@@ -1513,3 +1600,123 @@ clutter_event_get_axes (const ClutterEvent *event,
 
   return retval;
 }
+
+/**
+ * clutter_event_get_distance:
+ * @source: a #ClutterEvent
+ * @target: a #ClutterEvent
+ *
+ * Retrieves the distance between two events, a @source and a @target.
+ *
+ * Return value: the distance between two #ClutterEvent
+ *
+ * Since: 1.12
+ */
+float
+clutter_event_get_distance (const ClutterEvent *source,
+                            const ClutterEvent *target)
+{
+  ClutterPoint p0, p1;
+
+  clutter_event_get_position (source, &p0);
+  clutter_event_get_position (source, &p1);
+
+  return clutter_point_distance (&p0, &p1, NULL, NULL);
+}
+
+/**
+ * clutter_event_get_angle:
+ * @source: a #ClutterEvent
+ * @target: a #ClutterEvent
+ *
+ * Retrieves the angle relative from @source to @target.
+ *
+ * The direction of the angle is from the position X axis towards
+ * the positive Y axis.
+ *
+ * Return value: the angle between two #ClutterEvent
+ *
+ * Since: 1.12
+ */
+double
+clutter_event_get_angle (const ClutterEvent *source,
+                         const ClutterEvent *target)
+{
+  ClutterPoint p0, p1;
+  float x_distance, y_distance;
+  double angle;
+
+  clutter_event_get_position (source, &p0);
+  clutter_event_get_position (target, &p1);
+
+  if (clutter_point_equals (&p0, &p1))
+    return 0;
+
+  clutter_point_distance (&p0, &p1, &x_distance, &y_distance);
+
+  angle = atan2 (x_distance, y_distance);
+
+  /* invert the angle, and shift it by 90 degrees */
+  angle = (2.0 * G_PI) - angle;
+  angle += G_PI / 2.0;
+
+  /* keep the angle within the [ 0, 360 ] interval */
+  angle = fmod (angle, 2.0 * G_PI);
+
+  return angle;
+}
+
+/**
+ * clutter_event_has_shift_modifier:
+ * @event: a #ClutterEvent
+ *
+ * Checks whether @event has the Shift modifier mask set.
+ *
+ * Return value: %TRUE if the event has the Shift modifier mask set
+ *
+ * Since: 1.12
+ */
+gboolean
+clutter_event_has_shift_modifier (const ClutterEvent *event)
+{
+  return (clutter_event_get_state (event) & CLUTTER_SHIFT_MASK) != FALSE;
+}
+
+/**
+ * clutter_event_has_control_modifier:
+ * @event: a #ClutterEvent
+ *
+ * Checks whether @event has the Control modifier mask set.
+ *
+ * Return value: %TRUE if the event has the Control modifier mask set
+ *
+ * Since: 1.12
+ */
+gboolean
+clutter_event_has_control_modifier (const ClutterEvent *event)
+{
+  return (clutter_event_get_state (event) & CLUTTER_CONTROL_MASK) != FALSE;
+}
+
+/**
+ * clutter_event_is_pointer_emulated:
+ * @event: a #ClutterEvent
+ *
+ * Checks whether a pointer @event has been generated by the windowing
+ * system. The returned value can be used to distinguish between events
+ * synthesized by the windowing system itself (as opposed by Clutter).
+ *
+ * Return value: %TRUE if the event is pointer emulated
+ *
+ * Since: 1.12
+ */
+gboolean
+clutter_event_is_pointer_emulated (const ClutterEvent *event)
+{
+  g_return_val_if_fail (event != NULL, FALSE);
+
+  if (!is_event_allocated (event))
+    return FALSE;
+
+  return ((ClutterEventPrivate *) event)->is_pointer_emulated;
+}
