@@ -66,7 +66,7 @@ G_BEGIN_DECLS
 /**
  * CLUTTER_EVENT_STOP:
  *
- * Stops the propagattion of an event; this macro should be used
+ * Stops the propagation of an event; this macro should be used
  * in event-related signals.
  *
  * Since: 1.10
@@ -115,8 +115,6 @@ typedef struct _ClutterStageStateEvent  ClutterStageStateEvent;
 typedef struct _ClutterCrossingEvent    ClutterCrossingEvent;
 typedef struct _ClutterTouchEvent       ClutterTouchEvent;
 
-typedef struct _ClutterEventSequence    ClutterEventSequence;
-
 /**
  * ClutterAnyEvent:
  * @type: event type
@@ -148,7 +146,8 @@ struct _ClutterAnyEvent
  * @keyval: raw key value
  * @hardware_keycode: raw hardware key value
  * @unicode_value: Unicode representation
- * @device: reserved for future use
+ * @device: the device that originated the event. If you want the physical
+ * device the event originated from, use clutter_event_get_source_device()
  *
  * Key event
  *
@@ -183,7 +182,8 @@ struct _ClutterKeyEvent
  * @click_count: number of button presses within the default time
  *   and radius
  * @axes: reserved for future use
- * @device: reserved for future use
+ * @device: the device that originated the event. If you want the physical
+ * device the event originated from, use clutter_event_get_source_device()
  *
  * Button event.
  *
@@ -220,7 +220,8 @@ struct _ClutterButtonEvent
  * @x: event X coordinate
  * @y: event Y coordinate
  * @related: actor related to the crossing
- * @device: reserved for future use
+ * @device: the device that originated the event. If you want the physical
+ * device the event originated from, use clutter_event_get_source_device()
  *
  * Event for the movement of the pointer across different actors
  *
@@ -251,7 +252,8 @@ struct _ClutterCrossingEvent
  * @y: event Y coordinate
  * @modifier_state: button modifiers
  * @axes: reserved for future use
- * @device: reserved for future use
+ * @device: the device that originated the event. If you want the physical
+ * device the event originated from, use clutter_event_get_source_device()
  *
  * Event for the pointer motion
  *
@@ -284,7 +286,8 @@ struct _ClutterMotionEvent
  * @direction: direction of the scrolling
  * @modifier_state: button modifiers
  * @axes: reserved for future use
- * @device: reserved for future use
+ * @device: the device that originated the event. If you want the physical
+ * device the event originated from, use clutter_event_get_source_device()
  *
  * Scroll wheel (or similar device) event
  *
@@ -346,7 +349,8 @@ struct _ClutterStageStateEvent
  *   of modifier keys (e.g. Control, Shift, and Alt) and the pointer
  *   buttons. See #ClutterModifierType
  * @axes: reserved 
- * @device: the device that originated the event
+ * @device: the device that originated the event. If you want the physical
+ * device the event originated from, use clutter_event_get_source_device()
  *
  * Used for touch events.
  *
@@ -422,6 +426,13 @@ guint32                 clutter_event_get_time                  (const ClutterEv
 void                    clutter_event_set_state                 (ClutterEvent           *event,
                                                                  ClutterModifierType     state);
 ClutterModifierType     clutter_event_get_state                 (const ClutterEvent     *event);
+CLUTTER_AVAILABLE_IN_1_16
+void                    clutter_event_get_state_full            (const ClutterEvent     *event,
+								 ClutterModifierType    *button_state,
+								 ClutterModifierType    *base_state,
+								 ClutterModifierType    *latched_state,
+								 ClutterModifierType    *locked_state,
+								 ClutterModifierType    *effective_state);
 void                    clutter_event_set_device                (ClutterEvent           *event,
                                                                  ClutterInputDevice     *device);
 ClutterInputDevice *    clutter_event_get_device                (const ClutterEvent     *event);
@@ -445,10 +456,26 @@ void                    clutter_event_set_coords                (ClutterEvent
 void                    clutter_event_get_coords                (const ClutterEvent     *event,
                                                                  gfloat                 *x,
                                                                  gfloat                 *y);
+CLUTTER_AVAILABLE_IN_1_12
+void                    clutter_event_get_position              (const ClutterEvent     *event,
+                                                                 ClutterPoint           *position);
+CLUTTER_AVAILABLE_IN_1_12
+float                   clutter_event_get_distance              (const ClutterEvent     *source,
+                                                                 const ClutterEvent     *target);
+CLUTTER_AVAILABLE_IN_1_12
+double                  clutter_event_get_angle                 (const ClutterEvent     *source,
+                                                                 const ClutterEvent     *target);
 
 gdouble *               clutter_event_get_axes                  (const ClutterEvent     *event,
                                                                  guint                  *n_axes);
 
+CLUTTER_AVAILABLE_IN_1_12
+gboolean                clutter_event_has_shift_modifier        (const ClutterEvent     *event);
+CLUTTER_AVAILABLE_IN_1_12
+gboolean                clutter_event_has_control_modifier      (const ClutterEvent     *event);
+CLUTTER_AVAILABLE_IN_1_12
+gboolean                clutter_event_is_pointer_emulated       (const ClutterEvent     *event);
+
 void                    clutter_event_set_key_symbol            (ClutterEvent           *event,
                                                                  guint                   key_sym);
 guint                   clutter_event_get_key_symbol            (const ClutterEvent     *event);
@@ -456,8 +483,8 @@ void                    clutter_event_set_key_code              (ClutterEvent
                                                                  guint16                 key_code);
 guint16                 clutter_event_get_key_code              (const ClutterEvent     *event);
 void                    clutter_event_set_key_unicode           (ClutterEvent           *event,
-                                                                 guint32                 key_unicode);
-guint32                 clutter_event_get_key_unicode           (const ClutterEvent     *event);
+                                                                 gunichar                key_unicode);
+gunichar                clutter_event_get_key_unicode           (const ClutterEvent     *event);
 
 void                    clutter_event_set_button                (ClutterEvent           *event,
                                                                  guint32                 button);
