@@ -31,7 +31,7 @@
  * image data.
  *
  * <informalexample><programlisting>
- * <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" parse="text" href="../../../../tests/interactive/test-image-box.c">
+ * <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" parse="text" href="../../../../examples/image-content.c">
  *   <xi:fallback>FIXME: MISSING XINCLUDE CONTENT</xi:fallback>
  * </xi:include>
  * </programlisting></informalexample>
@@ -62,6 +62,7 @@ struct _ClutterImagePrivate
 static void clutter_content_iface_init (ClutterContentIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (ClutterImage, clutter_image, G_TYPE_OBJECT,
+                         G_ADD_PRIVATE (ClutterImage)
                          G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_CONTENT,
                                                 clutter_content_iface_init))
 
@@ -88,16 +89,13 @@ clutter_image_finalize (GObject *gobject)
 static void
 clutter_image_class_init (ClutterImageClass *klass)
 {
-  g_type_class_add_private (klass, sizeof (ClutterImagePrivate));
-
   G_OBJECT_CLASS (klass)->finalize = clutter_image_finalize;
 }
 
 static void
 clutter_image_init (ClutterImage *self)
 {
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, CLUTTER_TYPE_IMAGE,
-                                            ClutterImagePrivate);
+  self->priv = clutter_image_get_instance_private (self);
 }
 
 static void
@@ -107,6 +105,7 @@ clutter_image_paint_content (ClutterContent   *content,
 {
   ClutterImagePrivate *priv = CLUTTER_IMAGE (content)->priv;
   ClutterScalingFilter min_f, mag_f;
+  ClutterContentRepeat repeat;
   ClutterPaintNode *node;
   ClutterActorBox box;
   ClutterColor color;
@@ -118,15 +117,36 @@ clutter_image_paint_content (ClutterContent   *content,
   clutter_actor_get_content_box (actor, &box);
   paint_opacity = clutter_actor_get_paint_opacity (actor);
   clutter_actor_get_content_scaling_filters (actor, &min_f, &mag_f);
+  repeat = clutter_actor_get_content_repeat (actor);
 
-  color.red = paint_opacity;
-  color.green = paint_opacity;
-  color.blue = paint_opacity;
+  /* ClutterTextureNode will premultiply the blend color, so we
+   * want it to be white with the paint opacity
+   */
+  color.red = 255;
+  color.green = 255;
+  color.blue = 255;
   color.alpha = paint_opacity;
 
   node = clutter_texture_node_new (priv->texture, &color, min_f, mag_f);
   clutter_paint_node_set_name (node, "Image");
-  clutter_paint_node_add_rectangle (node, &box);
+
+  if (repeat == CLUTTER_REPEAT_NONE)
+    clutter_paint_node_add_rectangle (node, &box);
+  else
+    {
+      float t_w = 1.f, t_h = 1.f;
+
+      if ((repeat & CLUTTER_REPEAT_X_AXIS) != FALSE)
+        t_w = (box.x2 - box.x1) / cogl_texture_get_width (priv->texture);
+
+      if ((repeat & CLUTTER_REPEAT_Y_AXIS) != FALSE)
+        t_h = (box.y2 - box.y1) / cogl_texture_get_height (priv->texture);
+
+      clutter_paint_node_add_texture_rectangle (node, &box,
+                                                0.f, 0.f,
+                                                t_w, t_h);
+    }
+
   clutter_paint_node_add_child (root, node);
   clutter_paint_node_unref (node);
 }
@@ -183,7 +203,7 @@ clutter_image_new (void)
  * @row_stride: the length of each row inside @data
  * @error: return location for a #GError, or %NULL
  *
- * Sets the image data to be display by @image.
+ * Sets the image data to be displayed by @image.
  *
  * If the image data was successfully loaded, the @image will be invalidated.
  *
@@ -235,6 +255,69 @@ clutter_image_set_data (ClutterImage     *image,
   return TRUE;
 }
 
+/**
+ * clutter_image_set_bytes:
+ * @image: a #ClutterImage
+ * @data: the image data, as a #GBytes
+ * @pixel_format: the Cogl pixel format of the image data
+ * @width: the width of the image data
+ * @height: the height of the image data
+ * @row_stride: the length of each row inside @data
+ * @error: return location for a #GError, or %NULL
+ *
+ * Sets the image data stored inside a #GBytes to be displayed by @image.
+ *
+ * If the image data was successfully loaded, the @image will be invalidated.
+ *
+ * In case of error, the @error value will be set, and this function will
+ * return %FALSE.
+ *
+ * The image data contained inside the #GBytes is copied in texture memory,
+ * and no additional reference is acquired on the @data.
+ *
+ * Return value: %TRUE if the image data was successfully loaded,
+ *   and %FALSE otherwise.
+ *
+ * Since: 1.12
+ */
+gboolean
+clutter_image_set_bytes (ClutterImage     *image,
+                         GBytes           *data,
+                         CoglPixelFormat   pixel_format,
+                         guint             width,
+                         guint             height,
+                         guint             row_stride,
+                         GError          **error)
+{
+  ClutterImagePrivate *priv;
+
+  g_return_val_if_fail (CLUTTER_IS_IMAGE (image), FALSE);
+  g_return_val_if_fail (data != NULL, FALSE);
+
+  priv = image->priv;
+
+  if (priv->texture != NULL)
+    cogl_object_unref (priv->texture);
+
+  priv->texture = cogl_texture_new_from_data (width, height,
+                                              COGL_TEXTURE_NONE,
+                                              pixel_format,
+                                              COGL_PIXEL_FORMAT_ANY,
+                                              row_stride,
+                                              g_bytes_get_data (data, NULL));
+  if (priv->texture == NULL)
+    {
+      g_set_error_literal (error, CLUTTER_IMAGE_ERROR,
+                           CLUTTER_IMAGE_ERROR_INVALID_DATA,
+                           _("Unable to load image data"));
+      return FALSE;
+    }
+
+  clutter_content_invalidate (CLUTTER_CONTENT (image));
+
+  return TRUE;
+}
+
 /**
  * clutter_image_set_area:
  * @image: a #ClutterImage
