@@ -45,16 +45,19 @@ G_BEGIN_DECLS
  * Generic representation of an input device. The actual contents of this
  * structure depend on the backend used.
  */
-typedef struct _ClutterInputDevice      ClutterInputDevice;
 typedef struct _ClutterInputDeviceClass ClutterInputDeviceClass;
 
 GType clutter_input_device_get_type (void) G_GNUC_CONST;
 
 ClutterInputDeviceType  clutter_input_device_get_device_type    (ClutterInputDevice  *device);
 gint                    clutter_input_device_get_device_id      (ClutterInputDevice  *device);
-void                    clutter_input_device_get_device_coords  (ClutterInputDevice  *device,
-                                                                 gint                *x,
-                                                                 gint                *y);
+
+CLUTTER_AVAILABLE_IN_1_12
+gboolean                clutter_input_device_get_coords        (ClutterInputDevice   *device,
+                                                                ClutterEventSequence *sequence,
+                                                                ClutterPoint         *point);
+CLUTTER_AVAILABLE_IN_1_16
+ClutterModifierType     clutter_input_device_get_modifier_state (ClutterInputDevice  *device);
 ClutterActor *          clutter_input_device_get_pointer_actor  (ClutterInputDevice  *device);
 ClutterStage *          clutter_input_device_get_pointer_stage  (ClutterInputDevice  *device);
 const gchar *           clutter_input_device_get_device_name    (ClutterInputDevice  *device);
@@ -97,6 +100,17 @@ void                    clutter_input_device_ungrab             (ClutterInputDev
 CLUTTER_AVAILABLE_IN_1_10
 ClutterActor *          clutter_input_device_get_grabbed_actor  (ClutterInputDevice  *device);
 
+CLUTTER_AVAILABLE_IN_1_12
+void                    clutter_input_device_sequence_grab      (ClutterInputDevice   *device,
+                                                                 ClutterEventSequence *sequence,
+                                                                 ClutterActor         *actor);
+CLUTTER_AVAILABLE_IN_1_12
+void                    clutter_input_device_sequence_ungrab    (ClutterInputDevice   *device,
+                                                                 ClutterEventSequence *sequence);
+CLUTTER_AVAILABLE_IN_1_12
+ClutterActor *          clutter_input_device_sequence_get_grabbed_actor (ClutterInputDevice   *device,
+                                                                         ClutterEventSequence *sequence);
+
 gboolean                clutter_input_device_keycode_to_evdev   (ClutterInputDevice *device,
                                                                  guint               hardware_keycode,
                                                                  guint              *evdev_keycode);
