@@ -58,16 +58,21 @@
 #include <gobject/gvaluecollector.h>
 
 #include "clutter-color.h"
-#include "clutter-fixed.h"
 #include "clutter-interval.h"
 #include "clutter-private.h"
 #include "clutter-units.h"
+#include "clutter-scriptable.h"
+#include "clutter-script-private.h"
+
+#include "deprecated/clutter-fixed.h"
 
 enum
 {
   PROP_0,
 
   PROP_VALUE_TYPE,
+  PROP_INITIAL,
+  PROP_FINAL,
 
   PROP_LAST
 };
@@ -83,8 +88,6 @@ enum
   N_VALUES
 };
 
-#define CLUTTER_INTERVAL_GET_PRIVATE(obj)  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CLUTTER_TYPE_INTERVAL, ClutterIntervalPrivate))
-
 struct _ClutterIntervalPrivate
 {
   GType value_type;
@@ -92,7 +95,14 @@ struct _ClutterIntervalPrivate
   GValue *values;
 };
 
-G_DEFINE_TYPE (ClutterInterval, clutter_interval, G_TYPE_INITIALLY_UNOWNED);
+static void clutter_scriptable_iface_init (ClutterScriptableIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (ClutterInterval,
+                         clutter_interval,
+                         G_TYPE_INITIALLY_UNOWNED,
+                         G_ADD_PRIVATE (ClutterInterval)
+                         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_SCRIPTABLE,
+                                                clutter_scriptable_iface_init));
 
 static gboolean
 clutter_interval_real_validate (ClutterInterval *interval,
@@ -133,6 +143,21 @@ clutter_interval_real_validate (ClutterInterval *interval,
       }
       break;
 
+    case G_TYPE_INT64:
+      {
+        GParamSpecInt64 *pspec_int = G_PARAM_SPEC_INT64 (pspec);
+        gint64 a, b;
+
+        a = b = 0;
+        clutter_interval_get_interval (interval, &a, &b);
+        if ((a >= pspec_int->minimum && a <= pspec_int->maximum) &&
+            (b >= pspec_int->minimum && b <= pspec_int->maximum))
+          return TRUE;
+        else
+          return FALSE;
+      }
+      break;
+
     case G_TYPE_UINT:
       {
         GParamSpecUInt *pspec_uint = G_PARAM_SPEC_UINT (pspec);
@@ -148,6 +173,36 @@ clutter_interval_real_validate (ClutterInterval *interval,
       }
       break;
 
+    case G_TYPE_UINT64:
+      {
+        GParamSpecUInt64 *pspec_int = G_PARAM_SPEC_UINT64 (pspec);
+        guint64 a, b;
+
+        a = b = 0;
+        clutter_interval_get_interval (interval, &a, &b);
+        if ((a >= pspec_int->minimum && a <= pspec_int->maximum) &&
+            (b >= pspec_int->minimum && b <= pspec_int->maximum))
+          return TRUE;
+        else
+          return FALSE;
+      }
+      break;
+
+    case G_TYPE_CHAR:
+      {
+        GParamSpecChar *pspec_char = G_PARAM_SPEC_CHAR (pspec);
+        guchar a, b;
+
+        a = b = 0;
+        clutter_interval_get_interval (interval, &a, &b);
+        if ((a >= pspec_char->minimum && a <= pspec_char->maximum) &&
+            (b >= pspec_char->minimum && b <= pspec_char->maximum))
+          return TRUE;
+        else
+          return FALSE;
+      }
+      break;
+
     case G_TYPE_UCHAR:
       {
         GParamSpecUChar *pspec_uchar = G_PARAM_SPEC_UCHAR (pspec);
@@ -163,6 +218,36 @@ clutter_interval_real_validate (ClutterInterval *interval,
       }
       break;
 
+    case G_TYPE_FLOAT:
+      {
+        GParamSpecFloat *pspec_flt = G_PARAM_SPEC_FLOAT (pspec);
+        float a, b;
+
+        a = b = 0.f;
+        clutter_interval_get_interval (interval, &a, &b);
+        if ((a >= pspec_flt->minimum && a <= pspec_flt->maximum) &&
+            (b >= pspec_flt->minimum && b <= pspec_flt->maximum))
+          return TRUE;
+        else
+          return FALSE;
+      }
+      break;
+
+    case G_TYPE_DOUBLE:
+      {
+        GParamSpecDouble *pspec_flt = G_PARAM_SPEC_DOUBLE (pspec);
+        double a, b;
+
+        a = b = 0;
+        clutter_interval_get_interval (interval, &a, &b);
+        if ((a >= pspec_flt->minimum && a <= pspec_flt->maximum) &&
+            (b >= pspec_flt->minimum && b <= pspec_flt->maximum))
+          return TRUE;
+        else
+          return FALSE;
+      }
+      break;
+
     case G_TYPE_BOOLEAN:
       return TRUE;
 
@@ -215,6 +300,21 @@ clutter_interval_real_compute_value (ClutterInterval *interval,
       }
       break;
 
+    case G_TYPE_CHAR:
+      {
+        gchar ia, ib, res;
+
+        ia = g_value_get_schar (initial);
+        ib = g_value_get_schar (final);
+
+        res = (factor * (ib - (gdouble) ia)) + ia;
+
+        g_value_set_schar (value, res);
+
+        retval = TRUE;
+      }
+      break;
+
     case G_TYPE_UINT:
       {
         guint ia, ib, res;
@@ -292,7 +392,7 @@ clutter_interval_real_compute_value (ClutterInterval *interval,
    * a warning with a hint to what could be done to fix that */
   if (G_UNLIKELY (retval == FALSE))
     {
-      g_warning ("%s: Could not compute progress between two %ss. You can "
+      g_warning ("%s: Could not compute progress between two %s. You can "
                  "register a progress function to instruct ClutterInterval "
                  "how to deal with this GType",
                  G_STRLOC,
@@ -327,7 +427,8 @@ clutter_interval_set_property (GObject      *gobject,
                                const GValue *value,
                                GParamSpec   *pspec)
 {
-  ClutterIntervalPrivate *priv = CLUTTER_INTERVAL_GET_PRIVATE (gobject);
+  ClutterInterval *self = CLUTTER_INTERVAL (gobject);
+  ClutterIntervalPrivate *priv = clutter_interval_get_instance_private (self);
 
   switch (prop_id)
     {
@@ -335,6 +436,20 @@ clutter_interval_set_property (GObject      *gobject,
       priv->value_type = g_value_get_gtype (value);
       break;
 
+    case PROP_INITIAL:
+      if (g_value_get_boxed (value) != NULL)
+        clutter_interval_set_initial_value (self, g_value_get_boxed (value));
+      else if (G_IS_VALUE (&priv->values[INITIAL]))
+        g_value_unset (&priv->values[INITIAL]);
+      break;
+
+    case PROP_FINAL:
+      if (g_value_get_boxed (value) != NULL)
+        clutter_interval_set_final_value (self, g_value_get_boxed (value));
+      else if (G_IS_VALUE (&priv->values[FINAL]))
+        g_value_unset (&priv->values[FINAL]);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -347,7 +462,9 @@ clutter_interval_get_property (GObject    *gobject,
                                GValue     *value,
                                GParamSpec *pspec)
 {
-  ClutterIntervalPrivate *priv = CLUTTER_INTERVAL_GET_PRIVATE (gobject);
+  ClutterIntervalPrivate *priv;
+  
+  priv = clutter_interval_get_instance_private (CLUTTER_INTERVAL (gobject));
 
   switch (prop_id)
     {
@@ -355,19 +472,67 @@ clutter_interval_get_property (GObject    *gobject,
       g_value_set_gtype (value, priv->value_type);
       break;
 
+    case PROP_INITIAL:
+      if (G_IS_VALUE (&priv->values[INITIAL]))
+        g_value_set_boxed (value, &priv->values[INITIAL]);
+      break;
+
+    case PROP_FINAL:
+      if (G_IS_VALUE (&priv->values[FINAL]))
+        g_value_set_boxed (value, &priv->values[FINAL]);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
     }
 }
 
+static gboolean
+clutter_interval_parse_custom_node (ClutterScriptable *scriptable,
+                                    ClutterScript     *script,
+                                    GValue            *value,
+                                    const gchar       *name,
+                                    JsonNode          *node)
+{
+  ClutterIntervalPrivate *priv = CLUTTER_INTERVAL (scriptable)->priv;
+
+  if ((strcmp (name, "initial") == 0) || (strcmp (name, "final") == 0))
+    {
+      g_value_init (value, priv->value_type);
+      return _clutter_script_parse_node (script, value, name, node, NULL);
+    }
+
+  return FALSE;
+}
+
+static void
+clutter_interval_set_custom_property (ClutterScriptable *scriptable,
+                                      ClutterScript     *script,
+                                      const gchar       *name,
+                                      const GValue      *value)
+{
+  ClutterInterval *self = CLUTTER_INTERVAL (scriptable);
+
+  if (strcmp (name, "initial") == 0)
+    clutter_interval_set_initial_value (self, value);
+  else if (strcmp (name, "final") == 0)
+    clutter_interval_set_final_value (self, value);
+  else
+    g_object_set_property (G_OBJECT (scriptable), name, value);
+}
+
+static void
+clutter_scriptable_iface_init (ClutterScriptableIface *iface)
+{
+  iface->parse_custom_node = clutter_interval_parse_custom_node;
+  iface->set_custom_property = clutter_interval_set_custom_property;
+}
+
 static void
 clutter_interval_class_init (ClutterIntervalClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-  GParamSpec *pspec;
-
-  g_type_class_add_private (klass, sizeof (ClutterIntervalPrivate));
 
   klass->validate = clutter_interval_real_validate;
   klass->compute_value = clutter_interval_real_compute_value;
@@ -383,24 +548,55 @@ clutter_interval_class_init (ClutterIntervalClass *klass)
    *
    * Since: 1.0
    */
-  pspec = g_param_spec_gtype ("value-type",
-                              P_("Value Type"),
-                              P_("The type of the values in the interval"),
-                              G_TYPE_NONE,
-                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
-  obj_props[PROP_VALUE_TYPE] = pspec;
-  g_object_class_install_property (gobject_class, PROP_VALUE_TYPE, pspec);
+  obj_props[PROP_VALUE_TYPE] =
+    g_param_spec_gtype ("value-type",
+                        P_("Value Type"),
+                        P_("The type of the values in the interval"),
+                        G_TYPE_NONE,
+                        G_PARAM_READWRITE |
+                        G_PARAM_CONSTRUCT_ONLY |
+                        G_PARAM_STATIC_STRINGS);
+
+  /**
+   * ClutterInterval:initial:
+   *
+   * The initial value of the interval.
+   *
+   * Since: 1.12
+   */
+  obj_props[PROP_INITIAL] =
+    g_param_spec_boxed ("initial",
+                        P_("Initial Value"),
+                        P_("Initial value of the interval"),
+                        G_TYPE_VALUE,
+                        G_PARAM_READWRITE |
+                        G_PARAM_STATIC_STRINGS);
+
+  /**
+   * ClutterInterval:final:
+   *
+   * The final value of the interval.
+   *
+   * Since: 1.12
+   */
+  obj_props[PROP_FINAL] =
+    g_param_spec_boxed ("final",
+                        P_("Final Value"),
+                        P_("Final value of the interval"),
+                        G_TYPE_VALUE,
+                        G_PARAM_READWRITE |
+                        G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (gobject_class, PROP_LAST, obj_props);
 }
 
 static void
 clutter_interval_init (ClutterInterval *self)
 {
-  ClutterIntervalPrivate *priv;
+  self->priv = clutter_interval_get_instance_private (self);
 
-  self->priv = priv = CLUTTER_INTERVAL_GET_PRIVATE (self);
-
-  priv->value_type = G_TYPE_INVALID;
-  priv->values = g_malloc0 (sizeof (GValue) * N_VALUES);
+  self->priv->value_type = G_TYPE_INVALID;
+  self->priv->values = g_malloc0 (sizeof (GValue) * N_VALUES);
 }
 
 static inline void
@@ -409,6 +605,7 @@ clutter_interval_set_value_internal (ClutterInterval *interval,
                                      const GValue    *value)
 {
   ClutterIntervalPrivate *priv = interval->priv;
+  GType value_type;
 
   g_assert (index_ >= INITIAL && index_ <= RESULT);
 
@@ -416,7 +613,39 @@ clutter_interval_set_value_internal (ClutterInterval *interval,
     g_value_unset (&priv->values[index_]);
 
   g_value_init (&priv->values[index_], priv->value_type);
-  g_value_copy (value, &priv->values[index_]);
+
+  value_type = G_VALUE_TYPE (value);
+  if (value_type != priv->value_type ||
+      !g_type_is_a (value_type, priv->value_type))
+    {
+      if (g_value_type_compatible (value_type, priv->value_type))
+        {
+          g_value_copy (value, &priv->values[index_]);
+          return;
+        }
+
+      if (g_value_type_transformable (value_type, priv->value_type))
+        {
+          GValue transform = G_VALUE_INIT;
+
+          g_value_init (&transform, priv->value_type);
+
+          if (g_value_transform (value, &transform))
+            g_value_copy (&transform, &priv->values[index_]);
+          else
+            {
+              g_warning ("%s: Unable to convert a value of type '%s' into "
+                         "the value type '%s' of the interval.",
+                         G_STRLOC,
+                         g_type_name (value_type),
+                         g_type_name (priv->value_type));
+            }
+
+          g_value_unset (&transform);
+        }
+    }
+  else
+    g_value_copy (value, &priv->values[index_]);
 }
 
 static inline void
@@ -573,8 +802,8 @@ out:
 /**
  * clutter_interval_new_with_values:
  * @gtype: the type of the values in the interval
- * @initial: a #GValue holding the initial value of the interval
- * @final: a #GValue holding the final value of the interval
+ * @initial: (allow-none): a #GValue holding the initial value of the interval
+ * @final: (allow-none): a #GValue holding the final value of the interval
  *
  * Creates a new #ClutterInterval of type @gtype, between @initial
  * and @final.
@@ -590,20 +819,15 @@ clutter_interval_new_with_values (GType         gtype,
                                   const GValue *initial,
                                   const GValue *final)
 {
-  ClutterInterval *retval;
-
   g_return_val_if_fail (gtype != G_TYPE_INVALID, NULL);
-  g_return_val_if_fail (initial != NULL, NULL);
-  g_return_val_if_fail (final != NULL, NULL);
-  g_return_val_if_fail (G_VALUE_TYPE (initial) == gtype, NULL);
-  g_return_val_if_fail (G_VALUE_TYPE (final) == gtype, NULL);
+  g_return_val_if_fail (initial == NULL || G_VALUE_TYPE (initial) == gtype, NULL);
+  g_return_val_if_fail (final == NULL || G_VALUE_TYPE (final) == gtype, NULL);
 
-  retval = g_object_new (CLUTTER_TYPE_INTERVAL, "value-type", gtype, NULL);
-
-  clutter_interval_set_initial_value (retval, initial);
-  clutter_interval_set_final_value (retval, final);
-
-  return retval;
+  return g_object_new (CLUTTER_TYPE_INTERVAL,
+                       "value-type", gtype,
+                       "initial", initial,
+                       "final", final,
+                       NULL);
 }
 
 /**
@@ -672,15 +896,9 @@ void
 clutter_interval_set_initial_value (ClutterInterval *interval,
                                     const GValue    *value)
 {
-  ClutterIntervalPrivate *priv;
-
   g_return_if_fail (CLUTTER_IS_INTERVAL (interval));
   g_return_if_fail (value != NULL);
 
-  priv = interval->priv;
-
-  g_return_if_fail (G_VALUE_TYPE (value) == priv->value_type);
-
   clutter_interval_set_value_internal (interval, INITIAL, value);
 }
 
@@ -770,15 +988,9 @@ void
 clutter_interval_set_final_value (ClutterInterval *interval,
                                   const GValue    *value)
 {
-  ClutterIntervalPrivate *priv;
-
   g_return_if_fail (CLUTTER_IS_INTERVAL (interval));
   g_return_if_fail (value != NULL);
 
-  priv = interval->priv;
-
-  g_return_if_fail (G_VALUE_TYPE (value) == priv->value_type);
-
   clutter_interval_set_value_internal (interval, FINAL, value);
 }
 
@@ -1016,3 +1228,27 @@ clutter_interval_compute (ClutterInterval *interval,
 
   return NULL;
 }
+
+/**
+ * clutter_interval_is_valid:
+ * @interval: a #ClutterInterval
+ *
+ * Checks if the @interval has a valid initial and final values.
+ *
+ * Return value: %TRUE if the #ClutterInterval has an initial and
+ *   final values, and %FALSE otherwise
+ *
+ * Since: 1.12
+ */
+gboolean
+clutter_interval_is_valid (ClutterInterval *interval)
+{
+  ClutterIntervalPrivate *priv;
+
+  g_return_val_if_fail (CLUTTER_IS_INTERVAL (interval), FALSE);
+
+  priv = interval->priv;
+
+  return G_IS_VALUE (&priv->values[INITIAL]) &&
+         G_IS_VALUE (&priv->values[FINAL]);
+}
