@@ -126,7 +126,7 @@
  * Since: 1.10
  */
 #ifndef CLUTTER_VERSION_MIN_REQUIRED
-# define CLUTTER_VERSION_MIN_REQUIRED   (CLUTTER_VERSION_PREV_STABLE)
+# define CLUTTER_VERSION_MIN_REQUIRED   (CLUTTER_VERSION_CUR_STABLE)
 #endif
 
 /**
@@ -248,24 +248,46 @@
 # define CLUTTER_AVAILABLE_IN_1_10
 #endif
 
-/**
- * CLUTTER_CAIRO_FORMAT_ARGB32:
- *
- * The #CoglPixelFormat to be used when uploading image data from
- * and to a Cairo image surface using %CAIRO_FORMAT_ARGB32 and
- * %CAIRO_FORMAT_RGB24 as #cairo_format_t.
- *
- * Since: 1.8
- */
-
-/* Cairo stores the data in native byte order as ARGB but Cogl's pixel
- * formats specify the actual byte order. Therefore we need to use a
- * different format depending on the architecture
- */
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-#define CLUTTER_CAIRO_FORMAT_ARGB32     (COGL_PIXEL_FORMAT_BGRA_8888_PRE)
+#if CLUTTER_VERSION_MIN_REQUIRED >= CLUTTER_VERSION_1_12
+# define CLUTTER_DEPRECATED_IN_1_12             CLUTTER_DEPRECATED
+# define CLUTTER_DEPRECATED_IN_1_12_FOR(f)      CLUTTER_DEPRECATED_FOR(f)
 #else
-#define CLUTTER_CAIRO_FORMAT_ARGB32     (COGL_PIXEL_FORMAT_ARGB_8888_PRE)
+# define CLUTTER_DEPRECATED_IN_1_12
+# define CLUTTER_DEPRECATED_IN_1_12_FOR(f)
+#endif
+
+#if CLUTTER_VERSION_MAX_ALLOWED < CLUTTER_VERSION_1_12
+# define CLUTTER_AVAILABLE_IN_1_12              CLUTTER_UNAVAILABLE(1, 12)
+#else
+# define CLUTTER_AVAILABLE_IN_1_12
+#endif
+
+#if CLUTTER_VERSION_MIN_REQUIRED >= CLUTTER_VERSION_1_14
+# define CLUTTER_DEPRECATED_IN_1_14             CLUTTER_DEPRECATED
+# define CLUTTER_DEPRECATED_IN_1_14_FOR(f)      CLUTTER_DEPRECATED_FOR(f)
+#else
+# define CLUTTER_DEPRECATED_IN_1_14
+# define CLUTTER_DEPRECATED_IN_1_14_FOR(f)
+#endif
+
+#if CLUTTER_VERSION_MAX_ALLOWED < CLUTTER_VERSION_1_14
+# define CLUTTER_AVAILABLE_IN_1_14              CLUTTER_UNAVAILABLE(1, 14)
+#else
+# define CLUTTER_AVAILABLE_IN_1_14
+#endif
+
+#if CLUTTER_VERSION_MIN_REQUIRED >= CLUTTER_VERSION_1_16
+# define CLUTTER_DEPRECATED_IN_1_16             CLUTTER_DEPRECATED
+# define CLUTTER_DEPRECATED_IN_1_16_FOR(f)      CLUTTER_DEPRECATED_FOR(f)
+#else
+# define CLUTTER_DEPRECATED_IN_1_16
+# define CLUTTER_DEPRECATED_IN_1_16_FOR(f)
+#endif
+
+#if CLUTTER_VERSION_MAX_ALLOWED < CLUTTER_VERSION_1_16
+# define CLUTTER_AVAILABLE_IN_1_16              CLUTTER_UNAVAILABLE(1, 16)
+#else
+# define CLUTTER_AVAILABLE_IN_1_16
 #endif
 
 #endif /* __CLUTTER_MACROS_H__ */
