@@ -37,11 +37,11 @@
  * <refsect2 id="ClutterShaderEffect-implementing">
  *   <title>Implementing a ClutterShaderEffect</title>
  *   <para>Creating a sub-class of #ClutterShaderEffect requires the
- *   overriding of the <function>paint_target()</function> virtual
+ *   overriding of the #ClutterOffscreenEffectClass.paint_target() virtual
  *   function from the #ClutterOffscreenEffect class as well as the
  *   <function>get_static_shader_source()</function> virtual from the
  *   #ClutterShaderEffect class.</para>
- *   <para>The <function>get_static_shader_source()</function>
+ *   <para>The #ClutterShaderEffectClass.get_static_shader_source()
  *   function should return a copy of the shader source to use. This
  *   function is only called once per subclass of #ClutterShaderEffect
  *   regardless of how many instances of the effect are created. The
@@ -171,6 +171,7 @@ static GParamSpec *obj_props[PROP_LAST];
 G_DEFINE_TYPE_WITH_CODE (ClutterShaderEffect,
                          clutter_shader_effect,
                          CLUTTER_TYPE_OFFSCREEN_EFFECT,
+                         G_ADD_PRIVATE (ClutterShaderEffect)
                          g_type_add_class_private (g_define_type_id,
                                                    sizeof (ClutterShaderEffectClassPrivate)))
 
@@ -470,8 +471,6 @@ clutter_shader_effect_class_init (ClutterShaderEffectClass *klass)
 
   offscreen_class = CLUTTER_OFFSCREEN_EFFECT_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (ClutterShaderEffectPrivate));
-
   /**
    * ClutterShaderEffect:shader-type:
    *
@@ -503,9 +502,7 @@ clutter_shader_effect_class_init (ClutterShaderEffectClass *klass)
 static void
 clutter_shader_effect_init (ClutterShaderEffect *effect)
 {
-  effect->priv = G_TYPE_INSTANCE_GET_PRIVATE (effect,
-                                              CLUTTER_TYPE_SHADER_EFFECT,
-                                              ClutterShaderEffectPrivate);
+  effect->priv = clutter_shader_effect_get_instance_private (effect);
 }
 
 /**
@@ -519,7 +516,7 @@ clutter_shader_effect_init (ClutterShaderEffect *effect)
  * The effect will be empty until clutter_shader_effect_set_shader_source()
  * is called.
  *
- * Return value: (transfer full): the newly created #ClutterShaderEffect.
+ * Return value: the newly created #ClutterShaderEffect.
  *   Use g_object_unref() when done.
  *
  * Since: 1.8
