@@ -84,6 +84,7 @@
 
 #define CLUTTER_DISABLE_DEPRECATION_WARNINGS
 #include "deprecated/clutter-container.h"
+#include "deprecated/clutter-alpha.h"
 
 #include "clutter-table-layout.h"
 
@@ -97,8 +98,6 @@
 #define CLUTTER_TABLE_CHILD(obj)          (G_TYPE_CHECK_INSTANCE_CAST ((obj), CLUTTER_TYPE_TABLE_CHILD, ClutterTableChild))
 #define CLUTTER_IS_TABLE_CHILD(obj)       (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CLUTTER_TYPE_TABLE_CHILD))
 
-#define CLUTTER_TABLE_LAYOUT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CLUTTER_TYPE_TABLE_LAYOUT, ClutterTableLayoutPrivate))
-
 typedef struct _ClutterTableChild         ClutterTableChild;
 typedef struct _ClutterLayoutMetaClass    ClutterTableChildClass;
 
@@ -139,12 +138,6 @@ struct _ClutterTableChild
 {
   ClutterLayoutMeta parent_instance;
 
-  /* the last stable allocation before an animation; it is used as the
-   * initial ActorBox when interpolating. If has_last_allocation is
-   * FALSE then this value is not yet known
-   */
-  ClutterActorBox last_allocation;
-
   gint col;
   gint row;
 
@@ -158,7 +151,6 @@ struct _ClutterTableChild
   guint y_expand            : 1;
   guint x_fill              : 1;
   guint y_fill              : 1;
-  guint has_last_allocation : 1;
 };
 
 enum
@@ -190,13 +182,9 @@ enum
 
 GType clutter_table_child_get_type (void);
 
-G_DEFINE_TYPE (ClutterTableChild,
-               clutter_table_child,
-               CLUTTER_TYPE_LAYOUT_META);
+G_DEFINE_TYPE (ClutterTableChild, clutter_table_child, CLUTTER_TYPE_LAYOUT_META)
 
-G_DEFINE_TYPE (ClutterTableLayout,
-               clutter_table_layout,
-               CLUTTER_TYPE_LAYOUT_MANAGER);
+G_DEFINE_TYPE_WITH_PRIVATE (ClutterTableLayout, clutter_table_layout, CLUTTER_TYPE_LAYOUT_MANAGER)
 
 /*
  * ClutterBoxChild
@@ -226,22 +214,12 @@ table_child_set_position (ClutterTableChild *self,
   if (row_changed || col_changed)
     {
       ClutterLayoutManager *layout;
-      ClutterTableLayoutPrivate *priv;
 
       layout = clutter_layout_meta_get_manager (CLUTTER_LAYOUT_META (self));
-      priv = CLUTTER_TABLE_LAYOUT (layout)->priv;
+      clutter_layout_manager_layout_changed (layout);
 
       g_object_freeze_notify (G_OBJECT (self));
 
-      if (priv->use_animations)
-        {
-          clutter_layout_manager_begin_animation (layout,
-                                                  priv->easing_duration,
-                                                  priv->easing_mode);
-        }
-      else
-        clutter_layout_manager_layout_changed (layout);
-
       if (row_changed)
         g_object_notify (G_OBJECT (self), "row");
 
@@ -276,19 +254,9 @@ table_child_set_span (ClutterTableChild  *self,
   if (row_changed || col_changed)
     {
       ClutterLayoutManager *layout;
-      ClutterTableLayout *table;
 
       layout = clutter_layout_meta_get_manager (CLUTTER_LAYOUT_META (self));
-      table = CLUTTER_TABLE_LAYOUT (layout);
-
-      if (table->priv->use_animations)
-        {
-          clutter_layout_manager_begin_animation (layout,
-                                                  table->priv->easing_duration,
-                                                  table->priv->easing_mode);
-        }
-      else
-        clutter_layout_manager_layout_changed (layout);
+      clutter_layout_manager_layout_changed (layout);
 
       if (row_changed)
         g_object_notify (G_OBJECT (self), "row-span");
@@ -322,25 +290,19 @@ table_child_set_align (ClutterTableChild     *self,
   if (x_changed || y_changed)
     {
       ClutterLayoutManager *layout;
-      ClutterTableLayout *table;
 
       layout = clutter_layout_meta_get_manager (CLUTTER_LAYOUT_META (self));
-      table = CLUTTER_TABLE_LAYOUT (layout);
+      clutter_layout_manager_layout_changed (layout);
 
-      if (table->priv->use_animations)
-        {
-          clutter_layout_manager_begin_animation (layout,
-                                                  table->priv->easing_duration,
-                                                  table->priv->easing_mode);
-        }
-      else
-        clutter_layout_manager_layout_changed (layout);
+      g_object_freeze_notify (G_OBJECT (self));
 
       if (x_changed)
         g_object_notify (G_OBJECT (self), "x-align");
 
       if (y_changed)
         g_object_notify (G_OBJECT (self), "y-align");
+
+      g_object_thaw_notify (G_OBJECT (self));
     }
 }
 
@@ -371,22 +333,12 @@ table_child_set_fill (ClutterTableChild *self,
   if (x_changed || y_changed)
     {
       ClutterLayoutManager *layout;
-      ClutterTableLayoutPrivate *priv;
 
       layout = clutter_layout_meta_get_manager (CLUTTER_LAYOUT_META (self));
-      priv = CLUTTER_TABLE_LAYOUT (layout)->priv;
+      clutter_layout_manager_layout_changed (layout);
 
       g_object_freeze_notify (G_OBJECT (self));
 
-      if (priv->use_animations)
-        {
-          clutter_layout_manager_begin_animation (layout,
-                                                  priv->easing_duration,
-                                                  priv->easing_mode);
-        }
-      else
-        clutter_layout_manager_layout_changed (layout);
-
       if (x_changed)
         g_object_notify (G_OBJECT (self), "x-fill");
 
@@ -424,22 +376,12 @@ table_child_set_expand (ClutterTableChild *self,
   if (x_changed || y_changed)
     {
       ClutterLayoutManager *layout;
-      ClutterTableLayoutPrivate *priv;
 
       layout = clutter_layout_meta_get_manager (CLUTTER_LAYOUT_META (self));
-      priv = CLUTTER_TABLE_LAYOUT (layout)->priv;
+      clutter_layout_manager_layout_changed (layout);
 
       g_object_freeze_notify (G_OBJECT (self));
 
-      if (priv->use_animations)
-        {
-          clutter_layout_manager_begin_animation (layout,
-                                                  priv->easing_duration,
-                                                  priv->easing_mode);
-        }
-      else
-        clutter_layout_manager_layout_changed (layout);
-
       if (x_changed)
         g_object_notify (G_OBJECT (self), "x-expand");
 
@@ -626,28 +568,28 @@ clutter_table_child_class_init (ClutterTableChildClass *klass)
   pspec = g_param_spec_boolean ("x-expand",
                                 P_("Horizontal Expand"),
                                 P_("Allocate extra space for the child in horizontal axis"),
-                                FALSE,
+                                TRUE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_CHILD_X_EXPAND, pspec);
 
   pspec = g_param_spec_boolean ("y-expand",
                                 P_("Vertical Expand"),
                                 P_("Allocate extra space for the child in vertical axis"),
-                                FALSE,
+                                TRUE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_CHILD_Y_EXPAND, pspec);
 
   pspec = g_param_spec_boolean ("x-fill",
                                 P_("Horizontal Fill"),
                                 P_("Whether the child should receive priority when the container is allocating spare space on the horizontal axis"),
-                                FALSE,
+                                TRUE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_CHILD_X_FILL, pspec);
 
   pspec = g_param_spec_boolean ("y-fill",
                                 P_("Vertical Fill"),
                                 P_("Whether the child should receive priority when the container is allocating spare space on the vertical axis"),
-                                FALSE,
+                                TRUE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_CHILD_Y_FILL, pspec);
 
@@ -696,8 +638,6 @@ clutter_table_child_init (ClutterTableChild *self)
 
   self->x_fill = TRUE;
   self->y_fill = TRUE;
-
-  self->has_last_allocation = FALSE;
 }
 
 static GType
@@ -762,6 +702,7 @@ calculate_col_widths (ClutterTableLayout *self,
   ClutterActor *actor, *child;
   gint i;
   DimensionData *columns;
+  ClutterOrientation orientation = CLUTTER_ORIENTATION_HORIZONTAL;
 
   update_row_col (self, container);
   g_array_set_size (priv->columns, 0);
@@ -812,7 +753,10 @@ calculate_col_widths (ClutterTableLayout *self,
       col->pref_size = MAX (col->pref_size, c_pref);
 
       if (!col->expand)
-        col->expand = meta->x_expand;
+        {
+          col->expand = clutter_actor_needs_expand (child, orientation) ||
+            meta->x_expand;
+        }
     }
 
   /* STAGE TWO: take spanning children into account */
@@ -861,7 +805,11 @@ calculate_col_widths (ClutterTableLayout *self,
             }
 
           if (!columns[i].expand)
-            columns[i].expand = meta->x_expand;
+            {
+              columns[i].expand = clutter_actor_needs_expand (child,
+                                                              orientation) ||
+                meta->x_expand;
+            }
         }
       min_width += priv->col_spacing * (meta->col_span - 1);
       pref_width += priv->col_spacing * (meta->col_span - 1);
@@ -935,8 +883,8 @@ calculate_col_widths (ClutterTableLayout *self,
             n_expand++;
         }
 
-      pref_width += priv->col_spacing * (priv->n_cols - 1);
-      min_width += priv->col_spacing * (priv->n_cols - 1);
+      pref_width += priv->col_spacing * MAX (priv->visible_cols - 1, 0);
+      min_width += priv->col_spacing * MAX (priv->visible_cols - 1, 0);
 
       if (for_width <= min_width)
         {
@@ -1029,6 +977,7 @@ calculate_row_heights (ClutterTableLayout *self,
   ClutterActor *actor, *child;
   gint i;
   DimensionData *rows, *columns;
+  ClutterOrientation orientation = CLUTTER_ORIENTATION_VERTICAL;
 
   update_row_col (self, container);
   g_array_set_size (priv->rows, 0);
@@ -1082,7 +1031,10 @@ calculate_row_heights (ClutterTableLayout *self,
       row->pref_size = MAX (row->pref_size, c_pref);
 
       if (!row->expand)
-        row->expand = meta->y_expand;
+        {
+          row->expand = clutter_actor_needs_expand (child, orientation) ||
+            meta->y_expand;
+        }
     }
 
   /* STAGE TWO: take spanning children into account */
@@ -1133,7 +1085,10 @@ calculate_row_heights (ClutterTableLayout *self,
             }
 
           if (!rows[i].expand)
-            rows[i].expand = meta->y_expand;
+            {
+              rows[i].expand = clutter_actor_needs_expand (child, orientation) ||
+                meta->y_expand;
+            }
         }
 
       min_height += priv->row_spacing * (meta->row_span - 1);
@@ -1218,8 +1173,8 @@ calculate_row_heights (ClutterTableLayout *self,
             n_expand++;
         }
 
-      pref_height += priv->row_spacing * (priv->n_rows - 1);
-      min_height += priv->row_spacing * (priv->n_rows - 1);
+      pref_height += priv->row_spacing * MAX (priv->visible_rows - 1, 0);
+      min_height += priv->row_spacing * MAX (priv->visible_rows - 1, 0);
 
       if (for_height <= min_height)
         {
@@ -1336,7 +1291,7 @@ clutter_table_layout_get_preferred_width (ClutterLayoutManager *layout,
   calculate_table_dimensions (self, container, -1, for_height);
   columns = (DimensionData *) (void *) priv->columns->data;
 
-  total_min_width = (priv->visible_cols - 1) * (float) priv->col_spacing;
+  total_min_width = MAX ((priv->visible_cols - 1) * (float) priv->col_spacing, 0);
   total_pref_width = total_min_width;
 
   for (i = 0; i < priv->n_cols; i++)
@@ -1376,7 +1331,7 @@ clutter_table_layout_get_preferred_height (ClutterLayoutManager *layout,
   calculate_table_dimensions (self, container, for_width, -1);
   rows = (DimensionData *) (void *) priv->rows->data;
 
-  total_min_height = (priv->visible_rows - 1) * (float) priv->row_spacing;
+  total_min_height = MAX ((priv->visible_rows - 1) * (float) priv->row_spacing, 0);
   total_pref_height = total_min_height;
 
   for (i = 0; i < self->priv->n_rows; i++)
@@ -1537,95 +1492,27 @@ clutter_table_layout_allocate (ClutterLayoutManager   *layout,
       childbox.y1 = (float) child_y;
       childbox.y2 = (float) MAX (0, child_y + row_height);
 
-      clutter_actor_allocate_align_fill (child, &childbox,
-                                         x_align, y_align,
-                                         x_fill, y_fill,
-                                         flags);
-
-      /* since we call this after allocate_align_fill(), this is
-       * just a cheap copy
-       */
-      clutter_actor_get_allocation_box (child, &childbox);
-
-      if (priv->use_animations && priv->is_animating)
+      if (priv->use_animations)
         {
-          ClutterActorBox *start = NULL;
-          ClutterActorBox end = { 0, };
-          gdouble p;
-
-          p = clutter_layout_manager_get_animation_progress (layout);
-
-          if (!meta->has_last_allocation)
-            {
-              /* if there is no allocation available then the child has just
-               * been added to the container; we put it in the final state
-               * and store its allocation for later
-               */
-              meta->last_allocation = childbox;
-              meta->has_last_allocation = TRUE;
-
-              goto do_allocate;
-            }
-
-          start = &meta->last_allocation;
-          end = childbox;
-
-          /* interpolate between the initial and final values */
-          clutter_actor_box_interpolate (start, &end, p, &childbox);
-
-          CLUTTER_NOTE (ANIMATION,
-                        "Animate { %.1f, %.1f, %.1f, %.1f }\t"
-                        "%.3f * { %.1f, %.1f, %.1f, %.1f }\t"
-                        "-> { %.1f, %.1f, %.1f, %.1f }",
-                        start->x1, start->y1,
-                        start->x2, start->y2,
-                        p,
-                        childbox.x1, childbox.y1,
-                        childbox.x2, childbox.y2,
-                        end.x1, end.y1,
-                        end.x2, end.y2);
+          clutter_actor_save_easing_state (child);
+          clutter_actor_set_easing_mode (child, priv->easing_mode);
+          clutter_actor_set_easing_duration (child, priv->easing_duration);
         }
+
+      if (clutter_actor_needs_expand (child, CLUTTER_ORIENTATION_HORIZONTAL) ||
+          clutter_actor_needs_expand (child, CLUTTER_ORIENTATION_VERTICAL))
+        clutter_actor_allocate (child, &childbox, flags);
       else
-        {
-          /* store the allocation for later animations */
-          meta->last_allocation = childbox;
-          meta->has_last_allocation = TRUE;
-        }
+        clutter_actor_allocate_align_fill (child, &childbox,
+                                           x_align, y_align,
+                                           x_fill, y_fill,
+                                           flags);
 
-    do_allocate:
-      clutter_actor_allocate (child, &childbox, flags);
+      if (priv->use_animations)
+        clutter_actor_restore_easing_state (child);
     }
 }
 
-static ClutterAlpha *
-clutter_table_layout_begin_animation (ClutterLayoutManager *manager,
-                                      guint                 duration,
-                                      gulong                easing)
-{
-  ClutterTableLayoutPrivate *priv = CLUTTER_TABLE_LAYOUT (manager)->priv;
-  ClutterLayoutManagerClass *parent_class;
-
-  priv->is_animating = TRUE;
-
-  /* we want the default implementation */
-  parent_class = CLUTTER_LAYOUT_MANAGER_CLASS (clutter_table_layout_parent_class);
-
-  return parent_class->begin_animation (manager, duration, easing);
-}
-
-static void
-clutter_table_layout_end_animation (ClutterLayoutManager *manager)
-{
-  ClutterTableLayoutPrivate *priv = CLUTTER_TABLE_LAYOUT (manager)->priv;
-  ClutterLayoutManagerClass *parent_class;
-
-  priv->is_animating = FALSE;
-
-  /* we want the default implementation */
-  parent_class = CLUTTER_LAYOUT_MANAGER_CLASS (clutter_table_layout_parent_class);
-  parent_class->end_animation (manager);
-}
-
 static void
 clutter_table_layout_set_property (GObject      *gobject,
                                    guint         prop_id,
@@ -1730,10 +1617,6 @@ clutter_table_layout_class_init (ClutterTableLayoutClass *klass)
   layout_class->set_container = clutter_table_layout_set_container;
   layout_class->get_child_meta_type =
     clutter_table_layout_get_child_meta_type;
-  layout_class->begin_animation = clutter_table_layout_begin_animation;
-  layout_class->end_animation = clutter_table_layout_end_animation;
-
-  g_type_class_add_private (klass, sizeof (ClutterTableLayoutPrivate));
 
   /**
    * ClutterTableLayout:column-spacing:
@@ -1767,9 +1650,18 @@ clutter_table_layout_class_init (ClutterTableLayoutClass *klass)
    * ClutterTableLayout:use-animations:
    *
    * Whether the #ClutterTableLayout should animate changes in the
-   * layout properties
+   * layout properties.
+   *
+   * By default, #ClutterTableLayout will honour the easing state of
+   * the children when allocating them. Setting this property to
+   * %TRUE will override the easing state with the layout manager's
+   * #ClutterTableLayout:easing-mode and #ClutterTableLayout:easing-duration
+   * properties.
    *
    * Since: 1.4
+   *
+   * Deprecated: 1.12: #ClutterTableLayout will honour the easing state
+   *   of the children when allocating them
    */
   pspec = g_param_spec_boolean ("use-animations",
                                 P_("Use Animations"),
@@ -1782,16 +1674,19 @@ clutter_table_layout_class_init (ClutterTableLayoutClass *klass)
    * ClutterTableLayout:easing-mode:
    *
    * The easing mode for the animations, in case
-   * #ClutterTableLayout:use-animations is set to %TRUE
+   * #ClutterTableLayout:use-animations is set to %TRUE.
    *
    * The easing mode has the same semantics of #ClutterAnimation:mode: it can
    * either be a value from the #ClutterAnimationMode enumeration, like
    * %CLUTTER_EASE_OUT_CUBIC, or a logical id as returned by
-   * clutter_alpha_register_func()
+   * clutter_alpha_register_func().
    *
-   * The default value is %CLUTTER_EASE_OUT_CUBIC
+   * The default value is %CLUTTER_EASE_OUT_CUBIC.
    *
    * Since: 1.4
+   *
+   * Deprecated: 1.12: #ClutterTableLayout will honour the easing state
+   *   of the children when allocating them
    */
   pspec = g_param_spec_ulong ("easing-mode",
                               P_("Easing Mode"),
@@ -1805,11 +1700,14 @@ clutter_table_layout_class_init (ClutterTableLayoutClass *klass)
    * ClutterTableLayout:easing-duration:
    *
    * The duration of the animations, in case #ClutterTableLayout:use-animations
-   * is set to %TRUE
+   * is set to %TRUE.
    *
-   * The duration is expressed in milliseconds
+   * The duration is expressed in milliseconds.
    *
    * Since: 1.4
+   *
+   * Deprecated: 1.12: #ClutterTableLayout will honour the easing state
+   *   of the children when allocating them
    */
   pspec = g_param_spec_uint ("easing-duration",
                              P_("Easing Duration"),
@@ -1825,7 +1723,7 @@ clutter_table_layout_init (ClutterTableLayout *layout)
 {
   ClutterTableLayoutPrivate *priv;
 
-  layout->priv = priv = CLUTTER_TABLE_LAYOUT_GET_PRIVATE (layout);
+  layout->priv = priv = clutter_table_layout_get_instance_private (layout);
 
   priv->row_spacing = 0;
   priv->col_spacing = 0;
@@ -1879,15 +1777,7 @@ clutter_table_layout_set_column_spacing (ClutterTableLayout *layout,
       priv->col_spacing = spacing;
 
       manager = CLUTTER_LAYOUT_MANAGER (layout);
-
-      if (priv->use_animations)
-        {
-          clutter_layout_manager_begin_animation (manager,
-                                                  priv->easing_duration,
-                                                  priv->easing_mode);
-        }
-      else
-        clutter_layout_manager_layout_changed (manager);
+      clutter_layout_manager_layout_changed (manager);
 
       g_object_notify (G_OBJECT (layout), "column-spacing");
     }
@@ -1937,15 +1827,7 @@ clutter_table_layout_set_row_spacing (ClutterTableLayout *layout,
       priv->row_spacing = spacing;
 
       manager = CLUTTER_LAYOUT_MANAGER (layout);
-
-      if (priv->use_animations)
-        {
-          clutter_layout_manager_begin_animation (manager,
-                                                  priv->easing_duration,
-                                                  priv->easing_mode);
-        }
-      else
-        clutter_layout_manager_layout_changed (manager);
+      clutter_layout_manager_layout_changed (manager);
 
       g_object_notify (G_OBJECT (layout), "row-spacing");
     }
@@ -2015,10 +1897,10 @@ clutter_table_layout_pack (ClutterTableLayout  *layout,
   g_assert (CLUTTER_IS_TABLE_CHILD (meta));
 
   if (row < 0)
-    row = priv->n_rows + 1;
+    row = priv->n_rows;
 
   if (column < 0)
-    column = priv->n_cols + 1;
+    column = priv->n_cols;
 
   table_child_set_position (CLUTTER_TABLE_CHILD (meta), column, row);
 }
@@ -2146,6 +2028,9 @@ clutter_table_layout_get_span (ClutterTableLayout *layout,
  * inside @layout
  *
  * Since: 1.4
+ *
+ * Deprecated: 1.12: Use clutter_actor_set_x_align() and
+ *   clutter_actor_set_y_align() instead.
  */
 void
 clutter_table_layout_set_alignment (ClutterTableLayout    *layout,
@@ -2201,6 +2086,9 @@ clutter_table_layout_set_alignment (ClutterTableLayout    *layout,
  * clutter_table_layout_set_alignment().
  *
  * Since: 1.4
+ *
+ * Deprecated: 1.12: Use clutter_actor_get_x_align() and
+ *   clutter_actor_get_y_align() instead.
  */
 void
 clutter_table_layout_get_alignment (ClutterTableLayout    *layout,
@@ -2259,6 +2147,9 @@ clutter_table_layout_get_alignment (ClutterTableLayout    *layout,
  * inside @layout
  *
  * Since: 1.4
+ *
+ * Deprecated: 1.12: Use clutter_actor_set_x_align() and
+ *   clutter_actor_set_y_align() instead.
  */
 void
 clutter_table_layout_set_fill (ClutterTableLayout *layout,
@@ -2313,6 +2204,9 @@ clutter_table_layout_set_fill (ClutterTableLayout *layout,
  * as set using clutter_table_layout_pack() or clutter_table_layout_set_fill()
  *
  * Since: 1.4
+ *
+ * Deprecated: 1.12: Use clutter_actor_get_x_align() and
+ *   clutter_actor_get_y_align() instead.
  */
 void
 clutter_table_layout_get_fill (ClutterTableLayout *layout,
@@ -2372,6 +2266,9 @@ clutter_table_layout_get_fill (ClutterTableLayout *layout,
  * inside @layout
  *
  * Since: 1.4
+ *
+ * Deprecated: 1.12: Use clutter_actor_set_x_expand() or
+ *   clutter_actor_set_y_expand() instead.
  */
 void
 clutter_table_layout_set_expand (ClutterTableLayout *layout,
@@ -2426,6 +2323,9 @@ clutter_table_layout_set_expand (ClutterTableLayout *layout,
  * as set using clutter_table_layout_pack() or clutter_table_layout_set_expand()
  *
  * Since: 1.4
+ *
+ * Deprecated: 1.12: Use clutter_actor_get_x_expand() and
+ *   clutter_actor_get_y_expand() instead.
  */
 void
 clutter_table_layout_get_expand (ClutterTableLayout *layout,
@@ -2485,6 +2385,9 @@ clutter_table_layout_get_expand (ClutterTableLayout *layout,
  * by the animations is controlled by clutter_table_layout_set_easing_mode()
  *
  * Since: 1.4
+ *
+ * Deprecated: 1.12: #ClutterTableLayout will honour the easing state
+ *   of the children when allocating them
  */
 void
 clutter_table_layout_set_use_animations (ClutterTableLayout *layout,
@@ -2516,6 +2419,8 @@ clutter_table_layout_set_use_animations (ClutterTableLayout *layout,
  * Return value: %TRUE if the animations should be used, %FALSE otherwise
  *
  * Since: 1.4
+ *
+ * Deprecated: 1.12
  */
 gboolean
 clutter_table_layout_get_use_animations (ClutterTableLayout *layout)
@@ -2538,6 +2443,9 @@ clutter_table_layout_get_use_animations (ClutterTableLayout *layout)
  * animations
  *
  * Since: 1.4
+ *
+ * Deprecated: 1.12: #ClutterTableLayout will honour the easing state
+ *   of the children when allocating them
  */
 void
 clutter_table_layout_set_easing_mode (ClutterTableLayout *layout,
@@ -2566,6 +2474,9 @@ clutter_table_layout_set_easing_mode (ClutterTableLayout *layout,
  * Return value: an easing mode
  *
  * Since: 1.4
+ *
+ * Deprecated: 1.12: #ClutterTableLayout will honour the easing state
+ *   of the children when allocating them
  */
 gulong
 clutter_table_layout_get_easing_mode (ClutterTableLayout *layout)
@@ -2588,6 +2499,9 @@ clutter_table_layout_get_easing_mode (ClutterTableLayout *layout)
  * animations
  *
  * Since: 1.4
+ *
+ * Deprecated: 1.12: #ClutterTableLayout will honour the easing state
+ *   of the children when allocating them
  */
 void
 clutter_table_layout_set_easing_duration (ClutterTableLayout *layout,
@@ -2616,6 +2530,8 @@ clutter_table_layout_set_easing_duration (ClutterTableLayout *layout,
  * Return value: the duration of the animations, in milliseconds
  *
  * Since: 1.4
+ *
+ * Deprecated: 1.12
  */
 guint
 clutter_table_layout_get_easing_duration (ClutterTableLayout *layout)
