@@ -50,10 +50,11 @@
  * signal.
  *
  * Initial state can be set up by using the #ClutterTimeline::started signal,
- * while final state can be set up by using the #ClutterTimeline::completed
+ * while final state can be set up by using the #ClutterTimeline::stopped
  * signal. The #ClutterTimeline guarantees the emission of at least a single
  * #ClutterTimeline::new-frame signal, as well as the emission of the
- * #ClutterTimeline::completed signal.
+ * #ClutterTimeline::completed signal every time the #ClutterTimeline reaches
+ * its #ClutterTimeline:duration.
  *
  * It is possible to connect to specific points in the timeline progress by
  * adding <emphasis>markers</emphasis> using clutter_timeline_add_marker_at_time()
@@ -62,7 +63,8 @@
  * Timelines can be made to loop once they reach the end of their duration, by
  * using clutter_timeline_set_repeat_count(); a looping timeline will still
  * emit the #ClutterTimeline::completed signal once it reaches the end of its
- * duration.
+ * duration at each repeat. If you want to be notified of the end of the last
+ * repeat, use the #ClutterTimeline::stopped signal.
  *
  * Timelines have a #ClutterTimeline:direction: the default direction is
  * %CLUTTER_TIMELINE_FORWARD, and goes from 0 to the duration; it is possible
@@ -110,12 +112,6 @@
 
 #include "deprecated/clutter-timeline.h"
 
-static void clutter_scriptable_iface_init (ClutterScriptableIface *iface);
-
-G_DEFINE_TYPE_WITH_CODE (ClutterTimeline, clutter_timeline, G_TYPE_OBJECT,
-                         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_SCRIPTABLE,
-                                                clutter_scriptable_iface_init));
-
 struct _ClutterTimelinePrivate
 {
   ClutterTimelineDirection direction;
@@ -148,6 +144,14 @@ struct _ClutterTimelinePrivate
   GDestroyNotify progress_notify;
   ClutterAnimationMode progress_mode;
 
+  /* step() parameters */
+  gint n_steps;
+  ClutterStepMode step_mode;
+
+  /* cubic-bezier() parameters */
+  ClutterPoint cb_1;
+  ClutterPoint cb_2;
+
   guint is_playing         : 1;
 
   /* If we've just started playing and haven't yet gotten
@@ -159,8 +163,14 @@ struct _ClutterTimelinePrivate
 
 typedef struct {
   gchar *name;
-  guint msecs;
   GQuark quark;
+
+  union {
+    guint msecs;
+    gdouble progress;
+  } data;
+
+  guint is_relative : 1;
 } TimelineMarker;
 
 enum
@@ -187,21 +197,44 @@ enum
   PAUSED,
   COMPLETED,
   MARKER_REACHED,
+  STOPPED,
 
   LAST_SIGNAL
 };
 
 static guint timeline_signals[LAST_SIGNAL] = { 0, };
 
+static void clutter_scriptable_iface_init (ClutterScriptableIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (ClutterTimeline, clutter_timeline, G_TYPE_OBJECT,
+                         G_ADD_PRIVATE (ClutterTimeline)
+                         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_SCRIPTABLE,
+                                                clutter_scriptable_iface_init))
+
 static TimelineMarker *
-timeline_marker_new (const gchar *name,
-                     guint        msecs)
+timeline_marker_new_time (const gchar *name,
+                          guint        msecs)
 {
-  TimelineMarker *marker = g_slice_new0 (TimelineMarker);
+  TimelineMarker *marker = g_slice_new (TimelineMarker);
 
   marker->name = g_strdup (name);
   marker->quark = g_quark_from_string (marker->name);
-  marker->msecs = msecs;
+  marker->is_relative = FALSE;
+  marker->data.msecs = msecs;
+
+  return marker;
+}
+
+static TimelineMarker *
+timeline_marker_new_progress (const gchar *name,
+                              gdouble      progress)
+{
+  TimelineMarker *marker = g_slice_new (TimelineMarker);
+
+  marker->name = g_strdup (name);
+  marker->quark = g_quark_from_string (marker->name);
+  marker->is_relative = TRUE;
+  marker->data.progress = CLAMP (progress, 0.0, 1.0);
 
   return marker;
 }
@@ -245,9 +278,16 @@ clutter_timeline_add_marker_internal (ClutterTimeline *timeline,
   old_marker = g_hash_table_lookup (priv->markers_by_name, marker->name);
   if (old_marker != NULL)
     {
+      guint msecs;
+
+      if (old_marker->is_relative)
+        msecs = old_marker->data.progress * priv->duration;
+      else
+        msecs = old_marker->data.msecs;
+
       g_warning ("A marker named '%s' already exists at time %d",
                  old_marker->name,
-                 old_marker->msecs);
+                 msecs);
       timeline_marker_free (marker);
       return;
     }
@@ -304,12 +344,13 @@ parse_timeline_markers (JsonArray *array,
   object = json_node_get_object (element);
 
   if (!(json_object_has_member (object, "name") &&
-        json_object_has_member (object, "time")))
+        (json_object_has_member (object, "time") ||
+         json_object_has_member (object, "progress"))))
     {
       g_warning ("The marker definition in a ClutterTimeline description "
-                 "must be an object with the 'name' and 'time' members, "
-                 "but the element %d of the 'markers' array does not have "
-                 "either",
+                 "must be an object with the 'name' and either the 'time' "
+                 "or the 'progress' members, but the element %d of the "
+                 "'markers' array does not have any of them.",
                  index_);
       return;
     }
@@ -322,8 +363,12 @@ parse_timeline_markers (JsonArray *array,
       markers = NULL;
     }
 
-  marker = timeline_marker_new (json_object_get_string_member (object, "name"),
-                                json_object_get_int_member (object, "time"));
+  if (json_object_has_member (object, "time"))
+    marker = timeline_marker_new_time (json_object_get_string_member (object, "name"),
+                                       json_object_get_int_member (object, "time"));
+  else
+    marker = timeline_marker_new_progress (json_object_get_string_member (object, "name"),
+                                           json_object_get_double_member (object, "progress"));
 
   markers = g_list_prepend (markers, marker);
 
@@ -531,8 +576,6 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (ClutterTimelinePrivate));
-
   /**
    * ClutterTimeline:loop:
    *
@@ -684,6 +727,10 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
    *
    * This signal will be emitted even if the #ClutterTimeline is set to be
    * repeating.
+   *
+   * If you want to get notification on whether the #ClutterTimeline has
+   * been stopped or has finished its run, including its eventual repeats,
+   * you should use the #ClutterTimeline::stopped signal instead.
    */
   timeline_signals[COMPLETED] =
     g_signal_new (I_("completed"),
@@ -766,18 +813,49 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
                   G_TYPE_NONE, 2,
                   G_TYPE_STRING,
                   G_TYPE_INT);
+  /**
+   * ClutterTimeline::stopped:
+   * @timeline: the #ClutterTimeline that emitted the signal
+   * @is_finished: %TRUE if the signal was emitted at the end of the
+   *   timeline.
+   *
+   * The #ClutterTimeline::stopped signal is emitted when the timeline
+   * has been stopped, either because clutter_timeline_stop() has been
+   * called, or because it has been exhausted.
+   *
+   * This is different from the #ClutterTimeline::completed signal,
+   * which gets emitted after every repeat finishes.
+   *
+   * If the #ClutterTimeline has is marked as infinitely repeating,
+   * this signal will never be emitted.
+   *
+   * Since: 1.12
+   */
+  timeline_signals[STOPPED] =
+    g_signal_new (I_("stopped"),
+		  G_TYPE_FROM_CLASS (object_class),
+		  G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (ClutterTimelineClass, stopped),
+		  NULL, NULL,
+		  _clutter_marshal_VOID__BOOLEAN,
+		  G_TYPE_NONE, 1,
+                  G_TYPE_BOOLEAN);
 }
 
 static void
 clutter_timeline_init (ClutterTimeline *self)
 {
-  ClutterTimelinePrivate *priv;
+  self->priv = clutter_timeline_get_instance_private (self);
 
-  self->priv = priv =
-    G_TYPE_INSTANCE_GET_PRIVATE (self, CLUTTER_TYPE_TIMELINE,
-                                 ClutterTimelinePrivate);
+  self->priv->progress_mode = CLUTTER_LINEAR;
 
-  priv->progress_mode = CLUTTER_LINEAR;
+  /* default steps() parameters are 1, end */
+  self->priv->n_steps = 1;
+  self->priv->step_mode = CLUTTER_STEP_MODE_END;
+
+  /* default cubic-bezier() paramereters are (0, 0, 1, 1) */
+  clutter_point_init (&self->priv->cb_1, 0, 0);
+  clutter_point_init (&self->priv->cb_2, 1, 1);
 }
 
 struct CheckIfMarkerHitClosure
@@ -808,8 +886,8 @@ have_passed_time (const struct CheckIfMarkerHitClosure *data,
 
       /* Otherwise it's just a simple test if the time is in range of
          the previous time and the new time */
-      return (msecs > data->new_time - data->delta
-              && msecs <= data->new_time);
+      return (msecs > data->new_time - data->delta &&
+              msecs <= data->new_time);
     }
   else
     {
@@ -822,8 +900,8 @@ have_passed_time (const struct CheckIfMarkerHitClosure *data,
 
       /* Otherwise it's just a simple test if the time is in range of
          the previous time and the new time */
-      return (msecs >= data->new_time
-              && msecs < data->new_time + data->delta);
+      return (msecs >= data->new_time &&
+              msecs < data->new_time + data->delta);
     }
 }
 
@@ -832,14 +910,21 @@ check_if_marker_hit (const gchar *name,
                      TimelineMarker *marker,
                      struct CheckIfMarkerHitClosure *data)
 {
-  if (have_passed_time (data, marker->msecs))
+  gint msecs;
+
+  if (marker->is_relative)
+    msecs = (gdouble) data->duration * marker->data.progress;
+  else
+    msecs = marker->data.msecs;
+
+  if (have_passed_time (data, msecs))
     {
       CLUTTER_NOTE (SCHEDULER, "Marker '%s' reached", name);
 
       g_signal_emit (data->timeline, timeline_signals[MARKER_REACHED],
                      marker->quark,
                      name,
-                     marker->msecs);
+                     msecs);
     }
 }
 
@@ -875,6 +960,8 @@ emit_frame_signal (ClutterTimeline *timeline)
   /* see bug https://bugzilla.gnome.org/show_bug.cgi?id=654066 */
   gint elapsed = (gint) priv->elapsed_time;
 
+  CLUTTER_NOTE (SCHEDULER, "Emitting ::new-frame signal on timeline[%p]", timeline);
+
   g_signal_emit (timeline, timeline_signals[NEW_FRAME], 0, elapsed);
 }
 
@@ -895,12 +982,13 @@ set_is_playing (ClutterTimeline *timeline,
   ClutterTimelinePrivate *priv = timeline->priv;
   ClutterMasterClock *master_clock;
 
-  is_playing = is_playing != FALSE;
+  is_playing = !!is_playing;
 
   if (is_playing == priv->is_playing)
     return;
 
   priv->is_playing = is_playing;
+
   master_clock = _clutter_master_clock_get_default ();
   if (priv->is_playing)
     {
@@ -909,9 +997,7 @@ set_is_playing (ClutterTimeline *timeline,
       priv->current_repeat = 0;
     }
   else
-    {
-      _clutter_master_clock_remove_timeline (master_clock, timeline);
-    }
+    _clutter_master_clock_remove_timeline (master_clock, timeline);
 }
 
 static gboolean
@@ -923,7 +1009,7 @@ clutter_timeline_do_frame (ClutterTimeline *timeline)
 
   g_object_ref (timeline);
 
-  CLUTTER_NOTE (SCHEDULER, "Timeline [%p] activated (cur: %ld)\n",
+  CLUTTER_NOTE (SCHEDULER, "Timeline [%p] activated (elapsed time: %ld)\n",
                 timeline,
                 (long) priv->elapsed_time);
 
@@ -1000,10 +1086,13 @@ clutter_timeline_do_frame (ClutterTimeline *timeline)
            * priv->repeat_count. Are we limiting the things that could be
            * done in the above new-frame signal handler?
            */
-	  set_is_playing (timeline, FALSE);
-        }
+          set_is_playing (timeline, FALSE);
 
-      g_signal_emit (timeline, timeline_signals[COMPLETED], 0);
+          g_signal_emit (timeline, timeline_signals[COMPLETED], 0);
+          g_signal_emit (timeline, timeline_signals[STOPPED], 0, TRUE);
+        }
+      else
+        g_signal_emit (timeline, timeline_signals[COMPLETED], 0);
 
       priv->current_repeat += 1;
 
@@ -1154,8 +1243,22 @@ clutter_timeline_pause (ClutterTimeline *timeline)
 void
 clutter_timeline_stop (ClutterTimeline *timeline)
 {
+  gboolean was_playing;
+
+  g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
+
+  /* we check the is_playing here because pause() will return immediately
+   * if the timeline wasn't playing, so we don't know if it was actually
+   * stopped, and yet we still don't want to emit a ::stopped signal if
+   * the timeline was not playing in the first place.
+   */
+  was_playing = timeline->priv->is_playing;
+
   clutter_timeline_pause (timeline);
   clutter_timeline_rewind (timeline);
+
+  if (was_playing)
+    g_signal_emit (timeline, timeline_signals[STOPPED], 0, FALSE);
 }
 
 /**
@@ -1577,7 +1680,25 @@ clutter_timeline_get_delta (ClutterTimeline *timeline)
   return timeline->priv->msecs_delta;
 }
 
-/*
+void
+_clutter_timeline_advance (ClutterTimeline *timeline,
+                           gint64           tick_time)
+{
+  ClutterTimelinePrivate *priv = timeline->priv;
+
+  g_object_ref (timeline);
+
+  priv->msecs_delta = tick_time;
+  priv->is_playing = TRUE;
+
+  clutter_timeline_do_frame (timeline);
+
+  priv->is_playing = FALSE;
+
+  g_object_unref (timeline);
+}
+
+/*< private >
  * clutter_timeline_do_tick
  * @timeline: a #ClutterTimeline
  * @tick_time: time of advance
@@ -1593,8 +1714,6 @@ _clutter_timeline_do_tick (ClutterTimeline *timeline,
 {
   ClutterTimelinePrivate *priv;
 
-  g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
-
   priv = timeline->priv;
 
   /* Check the is_playing variable before performing the timeline tick.
@@ -1639,6 +1758,42 @@ _clutter_timeline_do_tick (ClutterTimeline *timeline,
     }
 }
 
+/**
+ * clutter_timeline_add_marker:
+ * @timeline: a #ClutterTimeline
+ * @marker_name: the unique name for this marker
+ * @progress: the normalized value of the position of the martke
+ *
+ * Adds a named marker that will be hit when the timeline has reached
+ * the specified @progress.
+ *
+ * Markers are unique string identifiers for a given position on the
+ * timeline. Once @timeline reaches the given @progress of its duration,
+ * if will emit a ::marker-reached signal for each marker attached to
+ * that particular point.
+ *
+ * A marker can be removed with clutter_timeline_remove_marker(). The
+ * timeline can be advanced to a marker using
+ * clutter_timeline_advance_to_marker().
+ *
+ * See also: clutter_timeline_add_marker_at_time()
+ *
+ * Since: 1.14
+ */
+void
+clutter_timeline_add_marker (ClutterTimeline *timeline,
+                             const gchar     *marker_name,
+                             gdouble          progress)
+{
+  TimelineMarker *marker;
+
+  g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
+  g_return_if_fail (marker_name != NULL);
+
+  marker = timeline_marker_new_progress (marker_name, progress);
+  clutter_timeline_add_marker_internal (timeline, marker);
+}
+
 /**
  * clutter_timeline_add_marker_at_time:
  * @timeline: a #ClutterTimeline
@@ -1646,15 +1801,18 @@ _clutter_timeline_do_tick (ClutterTimeline *timeline,
  * @msecs: position of the marker in milliseconds
  *
  * Adds a named marker that will be hit when the timeline has been
- * running for @msecs milliseconds. Markers are unique string
- * identifiers for a given time. Once @timeline reaches
- * @msecs, it will emit a ::marker-reached signal for each marker
- * attached to that time.
+ * running for @msecs milliseconds.
+ *
+ * Markers are unique string identifiers for a given position on the
+ * timeline. Once @timeline reaches the given @msecs, it will emit
+ * a ::marker-reached signal for each marker attached to that position.
  *
  * A marker can be removed with clutter_timeline_remove_marker(). The
  * timeline can be advanced to a marker using
  * clutter_timeline_advance_to_marker().
  *
+ * See also: clutter_timeline_add_marker()
+ *
  * Since: 0.8
  */
 void
@@ -1668,12 +1826,13 @@ clutter_timeline_add_marker_at_time (ClutterTimeline *timeline,
   g_return_if_fail (marker_name != NULL);
   g_return_if_fail (msecs <= clutter_timeline_get_duration (timeline));
 
-  marker = timeline_marker_new (marker_name, msecs);
+  marker = timeline_marker_new_time (marker_name, msecs);
   clutter_timeline_add_marker_internal (timeline, marker);
 }
 
 struct CollectMarkersClosure
 {
+  guint duration;
   guint msecs;
   GArray *markers;
 };
@@ -1683,7 +1842,14 @@ collect_markers (const gchar *key,
                  TimelineMarker *marker,
                  struct CollectMarkersClosure *data)
 {
-  if (marker->msecs == data->msecs)
+  guint msecs;
+
+  if (marker->is_relative)
+    msecs = marker->data.progress * data->duration;
+  else
+    msecs = marker->data.msecs;
+
+  if (msecs == data->msecs)
     {
       gchar *name_copy = g_strdup (key);
       g_array_append_val (data->markers, name_copy);
@@ -1743,6 +1909,7 @@ clutter_timeline_list_markers (ClutterTimeline *timeline,
     {
       struct CollectMarkersClosure data;
 
+      data.duration = priv->duration;
       data.msecs = msecs;
       data.markers = g_array_new (TRUE, FALSE, sizeof (gchar *));
 
@@ -1780,6 +1947,7 @@ clutter_timeline_advance_to_marker (ClutterTimeline *timeline,
 {
   ClutterTimelinePrivate *priv;
   TimelineMarker *marker;
+  guint msecs;
 
   g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
   g_return_if_fail (marker_name != NULL);
@@ -1793,13 +1961,18 @@ clutter_timeline_advance_to_marker (ClutterTimeline *timeline,
     }
 
   marker = g_hash_table_lookup (priv->markers_by_name, marker_name);
-  if (!marker)
+  if (marker == NULL)
     {
       g_warning ("No marker named '%s' found.", marker_name);
       return;
     }
 
-  clutter_timeline_advance (timeline, marker->msecs);
+  if (marker->is_relative)
+    msecs = marker->data.progress * priv->duration;
+  else
+    msecs = marker->data.msecs;
+
+  clutter_timeline_advance (timeline, msecs);
 }
 
 /**
@@ -2049,53 +2222,6 @@ clutter_timeline_set_progress_func (ClutterTimeline             *timeline,
   g_object_notify_by_pspec (G_OBJECT (timeline), obj_props[PROP_PROGRESS_MODE]);
 }
 
-/*< private >
- * _clutter_animation_modes:
- *
- * A mapping of animation modes and easing functions.
- */
-static const struct {
-  ClutterAnimationMode mode;
-  ClutterEasingFunc func;
-  const char *name;
-} _clutter_animation_modes[] = {
-  { CLUTTER_CUSTOM_MODE,         NULL, "custom" },
-
-  { CLUTTER_LINEAR,              clutter_linear, "linear" },
-  { CLUTTER_EASE_IN_QUAD,        clutter_ease_in_quad, "easeInQuad" },
-  { CLUTTER_EASE_OUT_QUAD,       clutter_ease_out_quad, "easeOutQuad" },
-  { CLUTTER_EASE_IN_OUT_QUAD,    clutter_ease_in_out_quad, "easeInOutQuad" },
-  { CLUTTER_EASE_IN_CUBIC,       clutter_ease_in_cubic, "easeInCubic" },
-  { CLUTTER_EASE_OUT_CUBIC,      clutter_ease_out_cubic, "easeOutCubic" },
-  { CLUTTER_EASE_IN_OUT_CUBIC,   clutter_ease_in_out_cubic, "easeInOutCubic" },
-  { CLUTTER_EASE_IN_QUART,       clutter_ease_in_quart, "easeInQuart" },
-  { CLUTTER_EASE_OUT_QUART,      clutter_ease_out_quart, "easeOutQuart" },
-  { CLUTTER_EASE_IN_OUT_QUART,   clutter_ease_in_out_quart, "easeInOutQuart" },
-  { CLUTTER_EASE_IN_QUINT,       clutter_ease_in_quint, "easeInQuint" },
-  { CLUTTER_EASE_OUT_QUINT,      clutter_ease_out_quint, "easeOutQuint" },
-  { CLUTTER_EASE_IN_OUT_QUINT,   clutter_ease_in_out_quint, "easeInOutQuint" },
-  { CLUTTER_EASE_IN_SINE,        clutter_ease_in_sine, "easeInSine" },
-  { CLUTTER_EASE_OUT_SINE,       clutter_ease_out_sine, "easeOutSine" },
-  { CLUTTER_EASE_IN_OUT_SINE,    clutter_ease_in_out_sine, "easeInOutSine" },
-  { CLUTTER_EASE_IN_EXPO,        clutter_ease_in_expo, "easeInExpo" },
-  { CLUTTER_EASE_OUT_EXPO,       clutter_ease_out_expo, "easeOutExpo" },
-  { CLUTTER_EASE_IN_OUT_EXPO,    clutter_ease_in_out_expo, "easeInOutExpo" },
-  { CLUTTER_EASE_IN_CIRC,        clutter_ease_in_circ, "easeInCirc" },
-  { CLUTTER_EASE_OUT_CIRC,       clutter_ease_out_circ, "easeOutCirc" },
-  { CLUTTER_EASE_IN_OUT_CIRC,    clutter_ease_in_out_circ, "easeInOutCirc" },
-  { CLUTTER_EASE_IN_ELASTIC,     clutter_ease_in_elastic, "easeInElastic" },
-  { CLUTTER_EASE_OUT_ELASTIC,    clutter_ease_out_elastic, "easeOutElastic" },
-  { CLUTTER_EASE_IN_OUT_ELASTIC, clutter_ease_in_out_elastic, "easeInOutElastic" },
-  { CLUTTER_EASE_IN_BACK,        clutter_ease_in_back, "easeInBack" },
-  { CLUTTER_EASE_OUT_BACK,       clutter_ease_out_back, "easeOutBack" },
-  { CLUTTER_EASE_IN_OUT_BACK,    clutter_ease_in_out_back, "easeInOutBack" },
-  { CLUTTER_EASE_IN_BOUNCE,      clutter_ease_in_bounce, "easeInBounce" },
-  { CLUTTER_EASE_OUT_BOUNCE,     clutter_ease_out_bounce, "easeOutBounce" },
-  { CLUTTER_EASE_IN_OUT_BOUNCE,  clutter_ease_in_out_bounce, "easeInOutBounce" },
-
-  { CLUTTER_ANIMATION_LAST,      NULL, "sentinel" },
-};
-
 static gdouble
 clutter_timeline_progress_func (ClutterTimeline *timeline,
                                 gdouble          elapsed,
@@ -2103,14 +2229,51 @@ clutter_timeline_progress_func (ClutterTimeline *timeline,
                                 gpointer         user_data G_GNUC_UNUSED)
 {
   ClutterTimelinePrivate *priv = timeline->priv;
-  ClutterEasingFunc easing_func;
 
-  g_assert (_clutter_animation_modes[priv->progress_mode].mode == priv->progress_mode);
-  g_assert (_clutter_animation_modes[priv->progress_mode].func != NULL);
+  /* parametrized easing functions need to be handled separately */
+  switch (priv->progress_mode)
+    {
+    case CLUTTER_STEPS:
+      if (priv->step_mode == CLUTTER_STEP_MODE_START)
+        return clutter_ease_steps_start (elapsed, duration, priv->n_steps);
+      else if (priv->step_mode == CLUTTER_STEP_MODE_END)
+        return clutter_ease_steps_end (elapsed, duration, priv->n_steps);
+      else
+        g_assert_not_reached ();
+      break;
 
-  easing_func = _clutter_animation_modes[priv->progress_mode].func;
+    case CLUTTER_STEP_START:
+      return clutter_ease_steps_start (elapsed, duration, 1);
 
-  return easing_func (elapsed, duration);
+    case CLUTTER_STEP_END:
+      return clutter_ease_steps_end (elapsed, duration, 1);
+
+    case CLUTTER_CUBIC_BEZIER:
+      return clutter_ease_cubic_bezier (elapsed, duration,
+                                        priv->cb_1.x, priv->cb_1.y,
+                                        priv->cb_2.x, priv->cb_2.y);
+
+    case CLUTTER_EASE:
+      return clutter_ease_cubic_bezier (elapsed, duration,
+                                        0.25, 0.1, 0.25, 1.0);
+
+    case CLUTTER_EASE_IN:
+      return clutter_ease_cubic_bezier (elapsed, duration,
+                                        0.42, 0.0, 1.0, 1.0);
+
+    case CLUTTER_EASE_OUT:
+      return clutter_ease_cubic_bezier (elapsed, duration,
+                                        0.0, 0.0, 0.58, 1.0);
+
+    case CLUTTER_EASE_IN_OUT:
+      return clutter_ease_cubic_bezier (elapsed, duration,
+                                        0.42, 0.0, 0.58, 1.0);
+
+    default:
+      break;
+    }
+
+  return clutter_easing_for_mode (priv->progress_mode, elapsed, duration);
 }
 
 /**
@@ -2228,3 +2391,148 @@ clutter_timeline_get_current_repeat (ClutterTimeline *timeline)
 
   return timeline->priv->current_repeat;
 }
+
+/**
+ * clutter_timeline_set_step_progress:
+ * @timeline: a #ClutterTimeline
+ * @n_steps: the number of steps
+ * @step_mode: whether the change should happen at the start
+ *   or at the end of the step
+ *
+ * Sets the #ClutterTimeline:progress-mode of the @timeline to %CLUTTER_STEPS
+ * and provides the parameters of the step function.
+ *
+ * Since: 1.12
+ */
+void
+clutter_timeline_set_step_progress (ClutterTimeline *timeline,
+                                    gint             n_steps,
+                                    ClutterStepMode  step_mode)
+{
+  ClutterTimelinePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
+  g_return_if_fail (n_steps > 0);
+
+  priv = timeline->priv;
+
+  if (priv->progress_mode == CLUTTER_STEPS &&
+      priv->n_steps == n_steps &&
+      priv->step_mode == step_mode)
+    return;
+
+  priv->n_steps = n_steps;
+  priv->step_mode = step_mode;
+  clutter_timeline_set_progress_mode (timeline, CLUTTER_STEPS);
+}
+
+/**
+ * clutter_timeline_get_step_progress:
+ * @timeline: a #ClutterTimeline
+ * @n_steps: (out): return location for the number of steps, or %NULL
+ * @step_mode: (out): return location for the value change policy,
+ *   or %NULL
+ *
+ * Retrieves the parameters of the step progress mode used by @timeline.
+ *
+ * Return value: %TRUE if the @timeline is using a step progress
+ *   mode, and %FALSE otherwise
+ *
+ * Since: 1.12
+ */
+gboolean
+clutter_timeline_get_step_progress (ClutterTimeline *timeline,
+                                    gint            *n_steps,
+                                    ClutterStepMode *step_mode)
+{
+  g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), FALSE);
+
+  if (timeline->priv->progress_mode != CLUTTER_STEPS ||
+      timeline->priv->progress_mode != CLUTTER_STEP_START ||
+      timeline->priv->progress_mode != CLUTTER_STEP_END)
+    return FALSE;
+
+  if (n_steps != NULL)
+    *n_steps = timeline->priv->n_steps;
+
+  if (step_mode != NULL)
+    *step_mode = timeline->priv->step_mode;
+
+  return TRUE;
+}
+
+/**
+ * clutter_timeline_set_cubic_bezier_progress:
+ * @timeline: a #ClutterTimeline
+ * @c_1: the first control point for the cubic bezier
+ * @c_2: the second control point for the cubic bezier
+ *
+ * Sets the #ClutterTimeline:progress-mode of @timeline
+ * to %CLUTTER_CUBIC_BEZIER, and sets the two control
+ * points for the cubic bezier.
+ *
+ * The cubic bezier curve is between (0, 0) and (1, 1). The X coordinate
+ * of the two control points must be in the [ 0, 1 ] range, while the
+ * Y coordinate of the two control points can exceed this range.
+ *
+ * Since: 1.12
+ */
+void
+clutter_timeline_set_cubic_bezier_progress (ClutterTimeline    *timeline,
+                                            const ClutterPoint *c_1,
+                                            const ClutterPoint *c_2)
+{
+  ClutterTimelinePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
+  g_return_if_fail (c_1 != NULL && c_2 != NULL);
+
+  priv = timeline->priv;
+
+  priv->cb_1 = *c_1;
+  priv->cb_2 = *c_2;
+
+  /* ensure the range on the X coordinate */
+  priv->cb_1.x = CLAMP (priv->cb_1.x, 0.f, 1.f);
+  priv->cb_2.x = CLAMP (priv->cb_2.x, 0.f, 1.f);
+
+  clutter_timeline_set_progress_mode (timeline, CLUTTER_CUBIC_BEZIER);
+}
+
+/**
+ * clutter_timeline_get_cubic_bezier_progress:
+ * @timeline: a #ClutterTimeline
+ * @c_1: (out caller-allocates): return location for the first control
+ *   point of the cubic bezier, or %NULL
+ * @c_2: (out caller-allocates): return location for the second control
+ *   point of the cubic bezier, or %NULL
+ *
+ * Retrieves the control points for the cubic bezier progress mode.
+ *
+ * Return value: %TRUE if the @timeline is using a cubic bezier progress
+ *   more, and %FALSE otherwise
+ *
+ * Since: 1.12
+ */
+gboolean
+clutter_timeline_get_cubic_bezier_progress (ClutterTimeline *timeline,
+                                            ClutterPoint    *c_1,
+                                            ClutterPoint    *c_2)
+{
+  g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), FALSE);
+
+  if (timeline->priv->progress_mode != CLUTTER_CUBIC_BEZIER ||
+      timeline->priv->progress_mode != CLUTTER_EASE ||
+      timeline->priv->progress_mode != CLUTTER_EASE_IN ||
+      timeline->priv->progress_mode != CLUTTER_EASE_OUT ||
+      timeline->priv->progress_mode != CLUTTER_EASE_IN_OUT)
+    return FALSE;
+
+  if (c_1 != NULL)
+    *c_1 = timeline->priv->cb_1;
+
+  if (c_2 != NULL)
+    *c_2 = timeline->priv->cb_2;
+
+  return TRUE;
+}
