@@ -128,9 +128,10 @@ cogl_gdk_filter (GdkXEvent  *xevent,
 		 gpointer    data)
 {
 #ifdef GDK_WINDOWING_X11
+  ClutterBackend *backend = data;
   CoglFilterReturn ret;
 
-  ret = cogl_xlib_handle_event ((XEvent*)xevent);
+  ret = cogl_xlib_renderer_handle_event (backend->cogl_renderer, (XEvent *) xevent);
   switch (ret)
     {
     case COGL_FILTER_REMOVE:
@@ -167,19 +168,10 @@ _clutter_backend_gdk_post_parse (ClutterBackend  *backend,
 
   g_assert (backend_gdk->display != NULL);
 
-#ifdef GDK_WINDOWING_X11
-  if (GDK_IS_X11_DISPLAY (backend_gdk->display))
-    {
-      /* Cogl needs to know the Xlib display connection for
-	 CoglTexturePixmapX11 */
-      cogl_xlib_set_display (gdk_x11_display_get_xdisplay (backend_gdk->display));
-    }
-#endif
-
   backend_gdk->screen = gdk_display_get_default_screen (backend_gdk->display);
 
   /* add event filter for Cogl events */
-  gdk_window_add_filter (NULL, cogl_gdk_filter, NULL);
+  gdk_window_add_filter (NULL, cogl_gdk_filter, backend_gdk);
 
   clutter_backend_gdk_init_settings (backend_gdk);
 
@@ -219,7 +211,7 @@ clutter_backend_gdk_finalize (GObject *gobject)
 {
   ClutterBackendGdk *backend_gdk = CLUTTER_BACKEND_GDK (gobject);
 
-  gdk_window_remove_filter (NULL, cogl_gdk_filter, NULL);
+  gdk_window_remove_filter (NULL, cogl_gdk_filter, backend_gdk);
   g_object_unref (backend_gdk->display);
 
   G_OBJECT_CLASS (clutter_backend_gdk_parent_class)->finalize (gobject);
@@ -287,6 +279,7 @@ clutter_backend_gdk_get_renderer (ClutterBackend  *backend,
     {
       /* Force a WGL winsys on windows */
       cogl_renderer_set_winsys_id (renderer, COGL_WINSYS_ID_WGL);
+      cogl_win32_renderer_set_event_retrieval_enabled (renderer, FALSE);
     }
   else
 #endif
