@@ -50,10 +50,6 @@
 #include <X11/extensions/Xcomposite.h>
 #endif
 
-#if HAVE_XINPUT
-#include <X11/extensions/XInput.h>
-#endif
-
 #if HAVE_XINPUT_2
 #include <X11/extensions/XInput2.h>
 #endif
@@ -94,7 +90,7 @@ static const gchar *atom_names[] = {
 
 /* various flags corresponding to pre init setup calls */
 static gboolean _no_xevent_retrieval = FALSE;
-static gboolean clutter_enable_xinput = FALSE;
+static gboolean clutter_enable_xinput = TRUE;
 static gboolean clutter_enable_argb = FALSE;
 static Display  *_foreign_dpy = NULL;
 
@@ -128,10 +124,11 @@ cogl_xlib_filter (XEvent       *xevent,
                   ClutterEvent *event,
                   gpointer      data)
 {
+  ClutterBackend *backend = data;
   ClutterX11FilterReturn retval;
   CoglFilterReturn ret;
 
-  ret = cogl_xlib_handle_event (xevent);
+  ret = cogl_xlib_renderer_handle_event (backend->cogl_renderer, xevent);
   switch (ret)
     {
     case COGL_FILTER_REMOVE:
@@ -168,7 +165,7 @@ clutter_backend_x11_xsettings_notify (const char       *name,
     {
       if (g_strcmp0 (name, CLUTTER_SETTING_X11_NAME (i)) == 0)
         {
-          GValue value = { 0, };
+          GValue value = G_VALUE_INIT;
 
           switch (setting->type)
             {
@@ -222,76 +219,55 @@ clutter_backend_x11_xsettings_notify (const char       *name,
 static void
 clutter_backend_x11_create_device_manager (ClutterBackendX11 *backend_x11)
 {
-  if (G_UNLIKELY (backend_x11->device_manager == NULL))
-    {
-      ClutterEventTranslator *translator;
-      ClutterBackend *backend;
+  ClutterEventTranslator *translator;
+  ClutterBackend *backend;
 
-#if defined(HAVE_XINPUT) || defined(HAVE_XINPUT_2)
-      if (clutter_enable_xinput)
-        {
-          int event_base, first_event, first_error;
-
-          if (XQueryExtension (backend_x11->xdpy, "XInputExtension",
-                               &event_base,
-                               &first_event,
-                               &first_error))
-            {
 #ifdef HAVE_XINPUT_2
-              int major = 2;
+  if (clutter_enable_xinput)
+    {
+      int event_base, first_event, first_error;
 
-#ifdef HAVE_XINPUT_2_2
-              int minor = 2;
-#else
-              int minor = 0;
-#endif /* HAVE_XINPUT_2_2 */
+      if (XQueryExtension (backend_x11->xdpy, "XInputExtension",
+                           &event_base,
+                           &first_event,
+                           &first_error))
+        {
+          int major = 2;
+          int minor = 3;
 
-              if (XIQueryVersion (backend_x11->xdpy, &major, &minor) != BadRequest)
-                {
-                  CLUTTER_NOTE (BACKEND, "Creating XI2 device manager");
-                  backend_x11->has_xinput = TRUE;
-                  backend_x11->device_manager =
-                    g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_XI2,
-                                  "backend", backend_x11,
-                                  "opcode", event_base,
-                                  NULL);
+          if (XIQueryVersion (backend_x11->xdpy, &major, &minor) != BadRequest)
+            {
+              CLUTTER_NOTE (BACKEND, "Creating XI2 device manager");
+              backend_x11->has_xinput = TRUE;
+              backend_x11->device_manager =
+                g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_XI2,
+                              "backend", backend_x11,
+                              "opcode", event_base,
+                              NULL);
 
-                  backend_x11->xi_minor = minor;
-                }
-              else
-#endif /* HAVE_XINPUT_2 */
-                {
-                  CLUTTER_NOTE (BACKEND, "Creating Core+XI device manager");
-                  backend_x11->has_xinput = TRUE;
-                  backend_x11->device_manager =
-                    g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_X11,
-                                  "backend", backend_x11,
-                                  "event-base", first_event,
-                                  NULL);
-
-                  backend_x11->xi_minor = -1;
-                }
+              backend_x11->xi_minor = minor;
             }
         }
-      else
-#endif /* HAVE_XINPUT || HAVE_XINPUT_2 */
-        {
-          CLUTTER_NOTE (BACKEND, "Creating Core device manager");
-          backend_x11->has_xinput = FALSE;
-          backend_x11->device_manager =
-            g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_X11,
-                          "backend", backend_x11,
-                          NULL);
-
-          backend_x11->xi_minor = -1;
-        }
-
-      backend = CLUTTER_BACKEND (backend_x11);
-      backend->device_manager = backend_x11->device_manager;
-
-      translator = CLUTTER_EVENT_TRANSLATOR (backend_x11->device_manager);
-      _clutter_backend_add_event_translator (backend, translator);
     }
+
+  if (backend_x11->device_manager == NULL)
+#endif /* HAVE_XINPUT_2 */
+    {
+      CLUTTER_NOTE (BACKEND, "Creating Core device manager");
+      backend_x11->has_xinput = FALSE;
+      backend_x11->device_manager =
+        g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_X11,
+                      "backend", backend_x11,
+                      NULL);
+
+      backend_x11->xi_minor = -1;
+    }
+
+  backend = CLUTTER_BACKEND (backend_x11);
+  backend->device_manager = backend_x11->device_manager;
+
+  translator = CLUTTER_EVENT_TRANSLATOR (backend_x11->device_manager);
+  _clutter_backend_add_event_translator (backend, translator);
 }
 
 static void
@@ -336,10 +312,10 @@ clutter_backend_x11_pre_parse (ClutterBackend  *backend,
       env_string = NULL;
     }
 
-  env_string = g_getenv ("CLUTTER_ENABLE_XINPUT");
+  env_string = g_getenv ("CLUTTER_DISABLE_XINPUT");
   if (env_string)
     {
-      clutter_enable_xinput = TRUE;
+      clutter_enable_xinput = FALSE;
       env_string = NULL;
     }
 
@@ -395,12 +371,8 @@ clutter_backend_x11_post_parse (ClutterBackend  *backend,
 
   settings = clutter_settings_get_default ();
 
-  /* Cogl needs to know the Xlib display connection for
-     CoglTexturePixmapX11 */
-  cogl_xlib_set_display (backend_x11->xdpy);
-
   /* add event filter for Cogl events */
-  clutter_x11_add_filter (cogl_xlib_filter, NULL);
+  clutter_x11_add_filter (cogl_xlib_filter, backend);
 
   if (clutter_screen == -1)
     backend_x11->xscreen = DefaultScreenOfDisplay (backend_x11->xdpy);
@@ -526,14 +498,14 @@ static const GOptionEntry entries[] =
     G_OPTION_ARG_NONE, &clutter_synchronise,
     N_("Make X calls synchronous"), NULL
   },
-#if defined(HAVE_XINPUT) || defined(HAVE_XINPUT_2)
+#ifdef HAVE_XINPUT_2
   {
-    "enable-xinput", 0,
-    0,
+    "disable-xinput", 0,
+    G_OPTION_FLAG_REVERSE,
     G_OPTION_ARG_NONE, &clutter_enable_xinput,
-    N_("Enable XInput support"), NULL
+    N_("Disable XInput support"), NULL
   },
-#endif /* HAVE_XINPUT */
+#endif /* HAVE_XINPUT_2 */
   { NULL }
 };
 
@@ -551,7 +523,7 @@ clutter_backend_x11_finalize (GObject *gobject)
 
   g_free (backend_x11->display_name);
 
-  clutter_x11_remove_filter (cogl_xlib_filter, NULL);
+  clutter_x11_remove_filter (cogl_xlib_filter, gobject);
 
   clutter_x11_remove_filter (xsettings_filter, backend_x11);
   _clutter_xsettings_client_destroy (backend_x11->xsettings);
@@ -782,7 +754,6 @@ clutter_backend_x11_create_stage (ClutterBackend  *backend,
 				  ClutterStage    *wrapper,
 				  GError         **error)
 {
-  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
   ClutterEventTranslator *translator;
   ClutterStageWindow *stage;
 
@@ -796,9 +767,9 @@ clutter_backend_x11_create_stage (ClutterBackend  *backend,
   _clutter_backend_add_event_translator (backend, translator);
 
   CLUTTER_NOTE (MISC, "X11 stage created (display:%p, screen:%d, root:%u)",
-                backend_x11->xdpy,
-                backend_x11->xscreen_num,
-                (unsigned int) backend_x11->xwin_root);
+                CLUTTER_BACKEND_X11 (backend)->xdpy,
+                CLUTTER_BACKEND_X11 (backend)->xscreen_num,
+                (unsigned int) CLUTTER_BACKEND_X11 (backend)->xwin_root);
 
   return stage;
 }
@@ -943,18 +914,15 @@ clutter_x11_set_display (Display *xdpy)
  * want to use clutter_x11_has_xinput() to see if support was enabled.
  *
  * Since: 0.8
+ *
+ * Deprecated: 1.14: This function does not do anything; XInput support
+ *   is enabled by default in Clutter. Use the CLUTTER_DISABLE_XINPUT
+ *   environment variable to disable XInput support and use Xlib core
+ *   events instead.
  */
 void
 clutter_x11_enable_xinput (void)
 {
-  if (_clutter_context_is_initialized ())
-    {
-      g_warning ("%s() can only be used before calling clutter_init()",
-                 G_STRFUNC);
-      return;
-    }
-
-  clutter_enable_xinput = TRUE;
 }
 
 /**
@@ -1185,6 +1153,8 @@ clutter_x11_get_input_devices (void)
   ClutterDeviceManager *manager;
 
   manager = clutter_device_manager_get_default ();
+  if (manager == NULL)
+    return NULL;
 
   return clutter_device_manager_peek_devices (manager);
 }
@@ -1202,7 +1172,7 @@ clutter_x11_get_input_devices (void)
 gboolean
 clutter_x11_has_xinput (void)
 {
-#if defined(HAVE_XINPUT) || defined(HAVE_XINPUT_2)
+#ifdef HAVE_XINPUT_2
  ClutterBackend *backend = clutter_get_default_backend ();
 
   if (backend == NULL)
