@@ -94,13 +94,13 @@ translate_valuator_class (Display             *xdisplay,
       atoms_initialized = TRUE;
     }
 
-  for (i = CLUTTER_INPUT_AXIS_IGNORE;
-       i < CLUTTER_INPUT_AXIS_LAST;
+  for (i = 0;
+       i < N_AXIS_ATOMS;
        i += 1)
     {
       if (clutter_input_axis_atoms[i] == class->label)
         {
-          axis = i;
+          axis = i + 1;
           break;
         }
     }
@@ -408,10 +408,16 @@ translate_hierarchy_event (ClutterBackendX11       *backend_x11,
 
           CLUTTER_NOTE (EVENT, "Hierarchy event: device enabled");
 
+          clutter_x11_trap_x_errors ();
           info = XIQueryDevice (backend_x11->xdpy,
                                 ev->info[i].deviceid,
                                 &n_devices);
-          add_device (manager_xi2, backend_x11, &info[0], FALSE);
+          clutter_x11_untrap_x_errors ();
+          if (info != NULL)
+            {
+              add_device (manager_xi2, backend_x11, &info[0], FALSE);
+              XIFreeDeviceInfo (info);
+            }
         }
       else if (ev->info[i].flags & XIDeviceDisabled)
         {
@@ -448,16 +454,24 @@ translate_hierarchy_event (ClutterBackendX11       *backend_x11,
           /* and attach the slave to the new master if needed */
           if (ev->info[i].flags & XISlaveAttached)
             {
+              clutter_x11_trap_x_errors ();
               info = XIQueryDevice (backend_x11->xdpy,
                                     ev->info[i].deviceid,
                                     &n_devices);
-              master = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                            GINT_TO_POINTER (info->attachment));
-              _clutter_input_device_set_associated_device (slave, master);
-              _clutter_input_device_add_slave (master, slave);
+              clutter_x11_untrap_x_errors ();
+              if (info != NULL)
+                {
+                  master = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                                GINT_TO_POINTER (info->attachment));
+                  if (master != NULL)
+                    {
+                      _clutter_input_device_set_associated_device (slave, master);
+                      _clutter_input_device_add_slave (master, slave);
 
-              send_changed = TRUE;
-              XIFreeDeviceInfo (info);
+                      send_changed = TRUE;
+                    }
+                  XIFreeDeviceInfo (info);
+                }
             }
 
           if (send_changed)
@@ -588,7 +602,6 @@ static gdouble *
 translate_axes (ClutterInputDevice *device,
                 gdouble             x,
                 gdouble             y,
-                ClutterStageX11    *stage_x11,
                 XIValuatorState    *valuators)
 {
   guint n_axes = clutter_input_device_get_n_axes (device);
@@ -629,6 +642,17 @@ translate_axes (ClutterInputDevice *device,
   return retval;
 }
 
+static void
+translate_coords (ClutterStageX11 *stage_x11,
+                  gdouble          event_x,
+                  gdouble          event_y,
+                  gfloat          *x_out,
+                  gfloat          *y_out)
+{
+  *x_out = CLAMP (event_x, 0, stage_x11->xwin_width)  / stage_x11->scale_factor;
+  *y_out = CLAMP (event_y, 0, stage_x11->xwin_height) / stage_x11->scale_factor;
+}
+
 static gdouble
 scroll_valuators_changed (ClutterInputDevice *device,
                           XIValuatorState    *valuators,
@@ -674,6 +698,49 @@ scroll_valuators_changed (ClutterInputDevice *device,
   return retval;
 }
 
+static void
+clutter_device_manager_xi2_select_stage_events (ClutterDeviceManager *manager,
+                                                ClutterStage         *stage)
+{
+  ClutterBackendX11 *backend_x11;
+  ClutterStageX11 *stage_x11;
+  XIEventMask xi_event_mask;
+  unsigned char *mask;
+  int len;
+
+  backend_x11 = CLUTTER_BACKEND_X11 (clutter_get_default_backend ());
+  stage_x11 = CLUTTER_STAGE_X11 (_clutter_stage_get_window (stage));
+
+  len = XIMaskLen (XI_LASTEVENT);
+  mask = g_new0 (unsigned char, len);
+
+  XISetMask (mask, XI_Motion);
+  XISetMask (mask, XI_ButtonPress);
+  XISetMask (mask, XI_ButtonRelease);
+  XISetMask (mask, XI_KeyPress);
+  XISetMask (mask, XI_KeyRelease);
+  XISetMask (mask, XI_Enter);
+  XISetMask (mask, XI_Leave);
+
+#ifdef HAVE_XINPUT_2_2
+  /* enable touch event support if we're running on XInput 2.2 */
+  if (backend_x11->xi_minor >= 2)
+    {
+      XISetMask (mask, XI_TouchBegin);
+      XISetMask (mask, XI_TouchUpdate);
+      XISetMask (mask, XI_TouchEnd);
+    }
+#endif /* HAVE_XINPUT_2_2 */
+
+  xi_event_mask.deviceid = XIAllMasterDevices;
+  xi_event_mask.mask = mask;
+  xi_event_mask.mask_len = len;
+
+  XISelectEvents (backend_x11->xdpy, stage_x11->xwin, &xi_event_mask, 1);
+
+  g_free (mask);
+}
+
 static ClutterTranslateReturn
 clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
                                             gpointer                native,
@@ -733,15 +800,19 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
 
         device = g_hash_table_lookup (manager_xi2->devices_by_id,
                                       GINT_TO_POINTER (xev->deviceid));
+        source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                             GINT_TO_POINTER (xev->sourceid));
         if (device)
           {
             _clutter_input_device_reset_axes (device);
-            _clutter_input_device_reset_scroll_info (device);
             translate_device_classes (backend_x11->xdpy,
                                       device,
                                       xev->classes,
                                       xev->num_classes);
           }
+
+        if (source_device)
+          _clutter_input_device_reset_scroll_info (source_device);
       }
       retval = CLUTTER_TRANSLATE_REMOVE;
       break;
@@ -760,15 +831,14 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
 
         event->key.time = xev->time;
         event->key.stage = stage;
-        event->key.modifier_state =
-          _clutter_input_device_xi2_translate_state (&xev->mods, &xev->buttons);
+	_clutter_input_device_xi2_translate_state (event, &xev->mods, &xev->buttons, &xev->group);
         event->key.hardware_keycode = xev->detail;
 
           /* keyval is the key ignoring all modifiers ('1' vs. '!') */
         event->key.keyval =
           _clutter_keymap_x11_translate_key_state (backend_x11->keymap,
                                                    event->key.hardware_keycode,
-                                                   event->key.modifier_state,
+                                                   &event->key.modifier_state,
                                                    NULL);
 
         /* KeyEvents have platform specific data associated to them */
@@ -867,11 +937,11 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
             event->scroll.stage = stage;
 
             event->scroll.time = xev->time;
-            event->scroll.x = xev->event_x;
-            event->scroll.y = xev->event_y;
-            event->scroll.modifier_state =
-              _clutter_input_device_xi2_translate_state (&xev->mods,
-                                                         &xev->buttons);
+            translate_coords (stage_x11, xev->event_x, xev->event_y, &event->scroll.x, &event->scroll.y);
+	    _clutter_input_device_xi2_translate_state (event,
+						       &xev->mods,
+						       &xev->buttons,
+						       &xev->group);
 
             clutter_event_set_source_device (event, source_device);
             clutter_event_set_device (event, device);
@@ -879,7 +949,6 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
             event->scroll.axes = translate_axes (event->scroll.device,
                                                  event->scroll.x,
                                                  event->scroll.y,
-                                                 stage_x11,
                                                  &xev->valuators);
 
             CLUTTER_NOTE (EVENT,
@@ -914,12 +983,12 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
             event->button.stage = stage;
 
             event->button.time = xev->time;
-            event->button.x = xev->event_x;
-            event->button.y = xev->event_y;
+            translate_coords (stage_x11, xev->event_x, xev->event_y, &event->button.x, &event->button.y);
             event->button.button = xev->detail;
-            event->button.modifier_state =
-              _clutter_input_device_xi2_translate_state (&xev->mods,
-                                                         &xev->buttons);
+	    _clutter_input_device_xi2_translate_state (event,
+						       &xev->mods,
+						       &xev->buttons,
+						       &xev->group);
 
             clutter_event_set_source_device (event, source_device);
             clutter_event_set_device (event, device);
@@ -927,7 +996,6 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
             event->button.axes = translate_axes (event->button.device,
                                                  event->button.x,
                                                  event->button.y,
-                                                 stage_x11,
                                                  &xev->valuators);
 
             CLUTTER_NOTE (EVENT,
@@ -996,11 +1064,11 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
 
             event->scroll.stage = stage;
             event->scroll.time = xev->time;
-            event->scroll.x = xev->event_x;
-            event->scroll.y = xev->event_y;
-            event->scroll.modifier_state =
-              _clutter_input_device_xi2_translate_state (&xev->mods,
-                                                         &xev->buttons);
+            translate_coords (stage_x11, xev->event_x, xev->event_y, &event->scroll.x, &event->scroll.y);
+	    _clutter_input_device_xi2_translate_state (event,
+						       &xev->mods,
+						       &xev->buttons,
+						       &xev->group);
 
             clutter_event_set_scroll_delta (event, delta_x, delta_y);
             clutter_event_set_source_device (event, source_device);
@@ -1024,11 +1092,11 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
         event->motion.stage = stage;
 
         event->motion.time = xev->time;
-        event->motion.x = xev->event_x;
-        event->motion.y = xev->event_y;
-        event->motion.modifier_state =
-          _clutter_input_device_xi2_translate_state (&xev->mods,
-                                                     &xev->buttons);
+        translate_coords (stage_x11, xev->event_x, xev->event_y, &event->motion.x, &event->motion.y);
+	_clutter_input_device_xi2_translate_state (event,
+						   &xev->mods,
+						   &xev->buttons,
+						   &xev->group);
 
         clutter_event_set_source_device (event, source_device);
         clutter_event_set_device (event, device);
@@ -1036,7 +1104,6 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
         event->motion.axes = translate_axes (event->motion.device,
                                              event->motion.x,
                                              event->motion.y,
-                                             stage_x11,
                                              &xev->valuators);
 
         if (source_device != NULL && device->stage != NULL)
@@ -1075,11 +1142,11 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
 
         event->touch.stage = stage;
         event->touch.time = xev->time;
-        event->touch.x = xev->event_x;
-        event->touch.y = xev->event_y;
-        event->touch.modifier_state =
-          _clutter_input_device_xi2_translate_state (&xev->mods,
-                                                     &xev->buttons);
+        translate_coords (stage_x11, xev->event_x, xev->event_y, &event->touch.x, &event->touch.y);
+	_clutter_input_device_xi2_translate_state (event,
+						   &xev->mods,
+						   &xev->buttons,
+						   &xev->group);
 
         clutter_event_set_source_device (event, source_device);
 
@@ -1090,7 +1157,6 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
         event->touch.axes = translate_axes (event->touch.device,
                                             event->motion.x,
                                             event->motion.y,
-                                            stage_x11,
                                             &xev->valuators);
 
         if (xi_event->evtype == XI_TouchBegin)
@@ -1130,8 +1196,7 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
         event->touch.stage = stage;
         event->touch.time = xev->time;
         event->touch.sequence = GUINT_TO_POINTER (xev->detail);
-        event->touch.x = xev->event_x;
-        event->touch.y = xev->event_y;
+        translate_coords (stage_x11, xev->event_x, xev->event_y, &event->touch.x, &event->touch.y);
 
         clutter_event_set_source_device (event, source_device);
 
@@ -1142,12 +1207,12 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
         event->touch.axes = translate_axes (event->touch.device,
                                             event->motion.x,
                                             event->motion.y,
-                                            stage_x11,
                                             &xev->valuators);
 
-        event->touch.modifier_state =
-          _clutter_input_device_xi2_translate_state (&xev->mods,
-                                                     &xev->buttons);
+	_clutter_input_device_xi2_translate_state (event,
+						   &xev->mods,
+						   &xev->buttons,
+						   &xev->group);
         event->touch.modifier_state |= CLUTTER_BUTTON1_MASK;
 
         if (xev->flags & XITouchEmulatingPointer)
@@ -1187,8 +1252,7 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
             event->crossing.related = NULL;
 
             event->crossing.time = xev->time;
-            event->crossing.x = xev->event_x;
-            event->crossing.y = xev->event_y;
+            translate_coords (stage_x11, xev->event_x, xev->event_y, &event->crossing.x, &event->crossing.y);
 
             _clutter_input_device_set_stage (device, stage);
           }
@@ -1211,8 +1275,7 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
             event->crossing.related = NULL;
 
             event->crossing.time = xev->time;
-            event->crossing.x = xev->event_x;
-            event->crossing.y = xev->event_y;
+            translate_coords (stage_x11, xev->event_x, xev->event_y, &event->crossing.x, &event->crossing.y);
 
             _clutter_input_device_set_stage (device, NULL);
           }
@@ -1341,8 +1404,8 @@ relate_slaves (gpointer key,
   ClutterDeviceManagerXI2 *manager_xi2 = data;
   ClutterInputDevice *master, *slave;
 
-  master = g_hash_table_lookup (manager_xi2->devices_by_id, key);
-  slave = g_hash_table_lookup (manager_xi2->devices_by_id, value);
+  slave = g_hash_table_lookup (manager_xi2->devices_by_id, key);
+  master = g_hash_table_lookup (manager_xi2->devices_by_id, value);
 
   _clutter_input_device_set_associated_device (slave, master);
   _clutter_input_device_add_slave (master, slave);
@@ -1459,6 +1522,7 @@ clutter_device_manager_xi2_class_init (ClutterDeviceManagerXI2Class *klass)
   manager_class->get_devices = clutter_device_manager_xi2_get_devices;
   manager_class->get_core_device = clutter_device_manager_xi2_get_core_device;
   manager_class->get_device = clutter_device_manager_xi2_get_device;
+  manager_class->select_stage_events = clutter_device_manager_xi2_select_stage_events;
 }
 
 static void
