@@ -15,23 +15,6 @@ test_conform_skip_test (TestConformSimpleFixture *fixture,
   /* void */
 }
 
-static void
-test_conform_todo_test (TestConformSimpleFixture *fixture,
-                        gconstpointer             data)
-{
-#ifdef G_OS_UNIX
-  const TestConformTodo *todo = data;
-
-  if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDOUT | G_TEST_TRAP_SILENCE_STDERR))
-    {
-      todo->func (fixture, NULL);
-      exit (0);
-    }
-
-  g_test_trap_assert_failed ();
-#endif
-}
-
 void
 verify_failure (TestConformSimpleFixture *fixture,
                 gconstpointer             data)
@@ -76,22 +59,10 @@ static TestConformSharedState *shared_state = NULL;
                 test_conform_simple_fixture_teardown);                  \
   }                                                     } G_STMT_END
 
-#define TEST_CONFORM_TODO(NAMESPACE, FUNC)              G_STMT_START {  \
-   extern void FUNC (TestConformSimpleFixture *, gconstpointer);        \
-   TestConformTodo *_clos = g_new0 (TestConformTodo, 1);                \
-   _clos->name = g_strdup ( #FUNC );                                    \
-   _clos->func = FUNC;                                                  \
-   g_test_add ("/todo" NAMESPACE "/" #FUNC,                             \
-              TestConformSimpleFixture,                                 \
-              _clos,                                                    \
-              test_conform_simple_fixture_setup,                        \
-              test_conform_todo_test,                                   \
-              test_conform_simple_fixture_teardown);    } G_STMT_END
-
 gchar *
 clutter_test_get_data_file (const gchar *filename)
 {
-  return g_build_filename (TESTS_DATADIR, filename, NULL);
+  return g_test_build_filename (G_TEST_DIST, "..", "data", filename, NULL);
 }
 
 static void
@@ -123,12 +94,9 @@ main (int argc, char **argv)
 
   /* This file is run through a sed script during the make step so the
      lines containing the tests need to be formatted on a single line
-     each. To comment out a test use the SKIP or TODO macros. Using
+     each. To comment out a test use the SKIP macro. Using
      #if 0 would break the script. */
 
-  /* sanity check for the test suite itself */
-  TEST_CONFORM_TODO ("/suite", verify_failure);
-
   TEST_CONFORM_SIMPLE ("/actor", actor_add_child);
   TEST_CONFORM_SIMPLE ("/actor", actor_insert_child);
   TEST_CONFORM_SIMPLE ("/actor", actor_raise_child);
@@ -145,6 +113,7 @@ main (int argc, char **argv)
   TEST_CONFORM_SIMPLE ("/actor", actor_basic_layout);
   TEST_CONFORM_SIMPLE ("/actor", actor_margin_layout);
   TEST_CONFORM_SIMPLE ("/actor", actor_offscreen_redirect);
+  TEST_CONFORM_SIMPLE ("/actor", actor_offscreen_limit_max_size);
   TEST_CONFORM_SIMPLE ("/actor", actor_shader_effect);
 
   TEST_CONFORM_SIMPLE ("/actor/iter", actor_iter_traverse_children);
@@ -164,6 +133,8 @@ main (int argc, char **argv)
   TEST_CONFORM_SIMPLE ("/actor/invariants", default_stage);
   TEST_CONFORM_SIMPLE ("/actor/invariants", actor_pivot_transformation);
 
+  TEST_CONFORM_SIMPLE ("/actor/meta", actor_meta_clear);
+
   TEST_CONFORM_SIMPLE ("/actor/opacity", opacity_label);
   TEST_CONFORM_SIMPLE ("/actor/opacity", opacity_rectangle);
   TEST_CONFORM_SIMPLE ("/actor/opacity", opacity_paint);
@@ -227,8 +198,9 @@ main (int argc, char **argv)
   TEST_CONFORM_SIMPLE ("/script", animator_multi_properties);
   TEST_CONFORM_SIMPLE ("/script", state_base);
   TEST_CONFORM_SIMPLE ("/script", script_margin);
+  TEST_CONFORM_SIMPLE ("/script", script_interval);
 
-  TEST_CONFORM_SIMPLE ("/timeline", timeline_base);
+  TEST_CONFORM_SKIP (g_test_slow (), "/timeline", timeline_base);
   TEST_CONFORM_SIMPLE ("/timeline", timeline_markers_from_script);
   TEST_CONFORM_SKIP (g_test_slow (), "/timeline", timeline_interpolation);
   TEST_CONFORM_SKIP (g_test_slow (), "/timeline", timeline_rewind);
@@ -241,30 +213,5 @@ main (int argc, char **argv)
 
   TEST_CONFORM_SIMPLE ("/events", events_touch);
 
-  /* FIXME - see bug https://bugzilla.gnome.org/show_bug.cgi?id=655588 */
-  TEST_CONFORM_TODO ("/cally", cally_text);
-
-  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_object);
-  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_fixed);
-  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_materials);
-  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_premult);
-  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_readpixels);
-
-  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_npot_texture);
-  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_multitexture);
-  TEST_CONFORM_TODO ("/cogl/texture", test_cogl_texture_mipmaps);
-  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_texture_rectangle);
-  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_texture_pixmap_x11);
-  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_texture_get_set_data);
-  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_atlas_migration);
-
-  TEST_CONFORM_SIMPLE ("/cogl/vertex-buffer", test_cogl_vertex_buffer_contiguous);
-  TEST_CONFORM_SIMPLE ("/cogl/vertex-buffer", test_cogl_vertex_buffer_interleved);
-  TEST_CONFORM_SIMPLE ("/cogl/vertex-buffer", test_cogl_vertex_buffer_mutability);
-
-  /* left to the end because they aren't currently very orthogonal and tend to
-   * break subsequent tests! */
-  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_viewport);
-
   return g_test_run ();
 }
